"""
Python module generated from Java source file java.time.Duration

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import DataInput
from java.io import DataOutput
from java.io import IOException
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.math import BigDecimal
from java.math import BigInteger
from java.math import RoundingMode
from java.time import *
from java.time.format import DateTimeParseException
from java.time.temporal import ChronoField
from java.time.temporal import ChronoUnit
from java.time.temporal import Temporal
from java.time.temporal import TemporalAmount
from java.time.temporal import TemporalUnit
from java.time.temporal import UnsupportedTemporalTypeException
from java.util import Objects
from java.util.regex import Matcher
from java.util.regex import Pattern
from typing import Any, Callable, Iterable, Tuple


class Duration(TemporalAmount, Comparable, Serializable):
    """
    A time-based amount of time, such as '34.5 seconds'.
    
    This class models a quantity or amount of time in terms of seconds and nanoseconds.
    It can be accessed using other duration-based units, such as minutes and hours.
    In addition, the ChronoUnit.DAYS DAYS unit can be used and is treated as
    exactly equal to 24 hours, thus ignoring daylight savings effects.
    See Period for the date-based equivalent to this class.
    
    A physical duration could be of infinite length.
    For practicality, the duration is stored with constraints similar to Instant.
    The duration uses nanosecond resolution with a maximum value of the seconds that can
    be held in a `long`. This is greater than the current estimated age of the universe.
    
    The range of a duration requires the storage of a number larger than a `long`.
    To achieve this, the class stores a `long` representing seconds and an `int`
    representing nanosecond-of-second, which will always be between 0 and 999,999,999.
    The model is of a directed duration, meaning that the duration may be negative.
    
    The duration is measured in "seconds", but these are not necessarily identical to
    the scientific "SI second" definition based on atomic clocks.
    This difference only impacts durations measured near a leap-second and should not affect
    most applications.
    See Instant for a discussion as to the meaning of the second and time-scales.
    
    This is a <a href="/java.base/java/lang/doc-files/ValueBased.html">value-based</a>
    class; programmers should treat instances that are
    .equals(Object) equal as interchangeable and should not
    use instances for synchronization, or unpredictable behavior may
    occur. For example, in a future release, synchronization may fail.
    The `equals` method should be used for comparisons.

    Since
    - 1.8

    Unknown Tags
    - This class is immutable and thread-safe.
    """

    ZERO = Duration(0, 0)
    """
    Constant for a duration of zero.
    """


    @staticmethod
    def ofDays(days: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of standard 24 hour days.
        
        The seconds are calculated based on the standard definition of a day,
        where each day is 86400 seconds which implies a 24 hour day.
        The nanosecond in second field is set to zero.

        Arguments
        - days: the number of days, positive or negative

        Returns
        - a `Duration`, not null

        Raises
        - ArithmeticException: if the input days exceeds the capacity of `Duration`
        """
        ...


    @staticmethod
    def ofHours(hours: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of standard hours.
        
        The seconds are calculated based on the standard definition of an hour,
        where each hour is 3600 seconds.
        The nanosecond in second field is set to zero.

        Arguments
        - hours: the number of hours, positive or negative

        Returns
        - a `Duration`, not null

        Raises
        - ArithmeticException: if the input hours exceeds the capacity of `Duration`
        """
        ...


    @staticmethod
    def ofMinutes(minutes: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of standard minutes.
        
        The seconds are calculated based on the standard definition of a minute,
        where each minute is 60 seconds.
        The nanosecond in second field is set to zero.

        Arguments
        - minutes: the number of minutes, positive or negative

        Returns
        - a `Duration`, not null

        Raises
        - ArithmeticException: if the input minutes exceeds the capacity of `Duration`
        """
        ...


    @staticmethod
    def ofSeconds(seconds: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of seconds.
        
        The nanosecond in second field is set to zero.

        Arguments
        - seconds: the number of seconds, positive or negative

        Returns
        - a `Duration`, not null
        """
        ...


    @staticmethod
    def ofSeconds(seconds: int, nanoAdjustment: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of seconds and an
        adjustment in nanoseconds.
        
        This method allows an arbitrary number of nanoseconds to be passed in.
        The factory will alter the values of the second and nanosecond in order
        to ensure that the stored nanosecond is in the range 0 to 999,999,999.
        For example, the following will result in exactly the same duration:
        ```
         Duration.ofSeconds(3, 1);
         Duration.ofSeconds(4, -999_999_999);
         Duration.ofSeconds(2, 1000_000_001);
        ```

        Arguments
        - seconds: the number of seconds, positive or negative
        - nanoAdjustment: the nanosecond adjustment to the number of seconds, positive or negative

        Returns
        - a `Duration`, not null

        Raises
        - ArithmeticException: if the adjustment causes the seconds to exceed the capacity of `Duration`
        """
        ...


    @staticmethod
    def ofMillis(millis: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of milliseconds.
        
        The seconds and nanoseconds are extracted from the specified milliseconds.

        Arguments
        - millis: the number of milliseconds, positive or negative

        Returns
        - a `Duration`, not null
        """
        ...


    @staticmethod
    def ofNanos(nanos: int) -> "Duration":
        """
        Obtains a `Duration` representing a number of nanoseconds.
        
        The seconds and nanoseconds are extracted from the specified nanoseconds.

        Arguments
        - nanos: the number of nanoseconds, positive or negative

        Returns
        - a `Duration`, not null
        """
        ...


    @staticmethod
    def of(amount: int, unit: "TemporalUnit") -> "Duration":
        """
        Obtains a `Duration` representing an amount in the specified unit.
        
        The parameters represent the two parts of a phrase like '6 Hours'. For example:
        ```
         Duration.of(3, SECONDS);
         Duration.of(465, HOURS);
        ```
        Only a subset of units are accepted by this method.
        The unit must either have an TemporalUnit.isDurationEstimated() exact duration or
        be ChronoUnit.DAYS which is treated as 24 hours. Other units throw an exception.

        Arguments
        - amount: the amount of the duration, measured in terms of the unit, positive or negative
        - unit: the unit that the duration is measured in, must have an exact duration, not null

        Returns
        - a `Duration`, not null

        Raises
        - DateTimeException: if the period unit has an estimated duration
        - ArithmeticException: if a numeric overflow occurs
        """
        ...


    @staticmethod
    def from(amount: "TemporalAmount") -> "Duration":
        """
        Obtains an instance of `Duration` from a temporal amount.
        
        This obtains a duration based on the specified amount.
        A `TemporalAmount` represents an  amount of time, which may be
        date-based or time-based, which this factory extracts to a duration.
        
        The conversion loops around the set of units from the amount and uses
        the TemporalUnit.getDuration() duration of the unit to
        calculate the total `Duration`.
        Only a subset of units are accepted by this method. The unit must either
        have an TemporalUnit.isDurationEstimated() exact duration
        or be ChronoUnit.DAYS which is treated as 24 hours.
        If any other units are found then an exception is thrown.

        Arguments
        - amount: the temporal amount to convert, not null

        Returns
        - the equivalent duration, not null

        Raises
        - DateTimeException: if unable to convert to a `Duration`
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    @staticmethod
    def parse(text: "CharSequence") -> "Duration":
        """
        Obtains a `Duration` from a text string such as `PnDTnHnMn.nS`.
        
        This will parse a textual representation of a duration, including the
        string produced by `toString()`. The formats accepted are based
        on the ISO-8601 duration format `PnDTnHnMn.nS` with days
        considered to be exactly 24 hours.
        
        The string starts with an optional sign, denoted by the ASCII negative
        or positive symbol. If negative, the whole period is negated.
        The ASCII letter "P" is next in upper or lower case.
        There are then four sections, each consisting of a number and a suffix.
        The sections have suffixes in ASCII of "D", "H", "M" and "S" for
        days, hours, minutes and seconds, accepted in upper or lower case.
        The suffixes must occur in order. The ASCII letter "T" must occur before
        the first occurrence, if any, of an hour, minute or second section.
        At least one of the four sections must be present, and if "T" is present
        there must be at least one section after the "T".
        The number part of each section must consist of one or more ASCII digits.
        The number may be prefixed by the ASCII negative or positive symbol.
        The number of days, hours and minutes must parse to a `long`.
        The number of seconds must parse to a `long` with optional fraction.
        The decimal point may be either a dot or a comma.
        The fractional part may have from zero to 9 digits.
        
        The leading plus/minus sign, and negative values for other units are
        not part of the ISO-8601 standard.
        
        Examples:
        ```
           "PT20.345S" -- parses as "20.345 seconds"
           "PT15M"     -- parses as "15 minutes" (where a minute is 60 seconds)
           "PT10H"     -- parses as "10 hours" (where an hour is 3600 seconds)
           "P2D"       -- parses as "2 days" (where a day is 24 hours or 86400 seconds)
           "P2DT3H4M"  -- parses as "2 days, 3 hours and 4 minutes"
           "PT-6H3M"    -- parses as "-6 hours and +3 minutes"
           "-PT6H3M"    -- parses as "-6 hours and -3 minutes"
           "-PT-6H+3M"  -- parses as "+6 hours and -3 minutes"
        ```

        Arguments
        - text: the text to parse, not null

        Returns
        - the parsed duration, not null

        Raises
        - DateTimeParseException: if the text cannot be parsed to a duration
        """
        ...


    @staticmethod
    def between(startInclusive: "Temporal", endExclusive: "Temporal") -> "Duration":
        """
        Obtains a `Duration` representing the duration between two temporal objects.
        
        This calculates the duration between two temporal objects. If the objects
        are of different types, then the duration is calculated based on the type
        of the first object. For example, if the first argument is a `LocalTime`
        then the second argument is converted to a `LocalTime`.
        
        The specified temporal objects must support the ChronoUnit.SECONDS SECONDS unit.
        For full accuracy, either the ChronoUnit.NANOS NANOS unit or the
        ChronoField.NANO_OF_SECOND NANO_OF_SECOND field should be supported.
        
        The result of this method can be a negative period if the end is before the start.
        To guarantee to obtain a positive duration call .abs() on the result.

        Arguments
        - startInclusive: the start instant, inclusive, not null
        - endExclusive: the end instant, exclusive, not null

        Returns
        - a `Duration`, not null

        Raises
        - DateTimeException: if the seconds between the temporals cannot be obtained
        - ArithmeticException: if the calculation exceeds the capacity of `Duration`
        """
        ...


    def get(self, unit: "TemporalUnit") -> int:
        """
        Gets the value of the requested unit.
        
        This returns a value for each of the two supported units,
        ChronoUnit.SECONDS SECONDS and ChronoUnit.NANOS NANOS.
        All other units throw an exception.

        Arguments
        - unit: the `TemporalUnit` for which to return the value

        Returns
        - the long value of the unit

        Raises
        - DateTimeException: if the unit is not supported
        - UnsupportedTemporalTypeException: if the unit is not supported
        """
        ...


    def getUnits(self) -> list["TemporalUnit"]:
        """
        Gets the set of units supported by this duration.
        
        The supported units are ChronoUnit.SECONDS SECONDS,
        and ChronoUnit.NANOS NANOS.
        They are returned in the order seconds, nanos.
        
        This set can be used in conjunction with .get(TemporalUnit)
        to access the entire state of the duration.

        Returns
        - a list containing the seconds and nanos units, not null
        """
        ...


    def isZero(self) -> bool:
        """
        Checks if this duration is zero length.
        
        A `Duration` represents a directed distance between two points on
        the time-line and can therefore be positive, zero or negative.
        This method checks whether the length is zero.

        Returns
        - True if this duration has a total length equal to zero
        """
        ...


    def isNegative(self) -> bool:
        """
        Checks if this duration is negative, excluding zero.
        
        A `Duration` represents a directed distance between two points on
        the time-line and can therefore be positive, zero or negative.
        This method checks whether the length is less than zero.

        Returns
        - True if this duration has a total length less than zero
        """
        ...


    def getSeconds(self) -> int:
        """
        Gets the number of seconds in this duration.
        
        The length of the duration is stored using two fields - seconds and nanoseconds.
        The nanoseconds part is a value from 0 to 999,999,999 that is an adjustment to
        the length in seconds.
        The total duration is defined by calling this method and .getNano().
        
        A `Duration` represents a directed distance between two points on the time-line.
        A negative duration is expressed by the negative sign of the seconds part.
        A duration of -1 nanosecond is stored as -1 seconds plus 999,999,999 nanoseconds.

        Returns
        - the whole seconds part of the length of the duration, positive or negative
        """
        ...


    def getNano(self) -> int:
        """
        Gets the number of nanoseconds within the second in this duration.
        
        The length of the duration is stored using two fields - seconds and nanoseconds.
        The nanoseconds part is a value from 0 to 999,999,999 that is an adjustment to
        the length in seconds.
        The total duration is defined by calling this method and .getSeconds().
        
        A `Duration` represents a directed distance between two points on the time-line.
        A negative duration is expressed by the negative sign of the seconds part.
        A duration of -1 nanosecond is stored as -1 seconds plus 999,999,999 nanoseconds.

        Returns
        - the nanoseconds within the second part of the length of the duration, from 0 to 999,999,999
        """
        ...


    def withSeconds(self, seconds: int) -> "Duration":
        """
        Returns a copy of this duration with the specified amount of seconds.
        
        This returns a duration with the specified seconds, retaining the
        nano-of-second part of this duration.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - seconds: the seconds to represent, may be negative

        Returns
        - a `Duration` based on this period with the requested seconds, not null
        """
        ...


    def withNanos(self, nanoOfSecond: int) -> "Duration":
        """
        Returns a copy of this duration with the specified nano-of-second.
        
        This returns a duration with the specified nano-of-second, retaining the
        seconds part of this duration.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - nanoOfSecond: the nano-of-second to represent, from 0 to 999,999,999

        Returns
        - a `Duration` based on this period with the requested nano-of-second, not null

        Raises
        - DateTimeException: if the nano-of-second is invalid
        """
        ...


    def plus(self, duration: "Duration") -> "Duration":
        """
        Returns a copy of this duration with the specified duration added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - duration: the duration to add, positive or negative, not null

        Returns
        - a `Duration` based on this duration with the specified duration added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plus(self, amountToAdd: int, unit: "TemporalUnit") -> "Duration":
        """
        Returns a copy of this duration with the specified duration added.
        
        The duration amount is measured in terms of the specified unit.
        Only a subset of units are accepted by this method.
        The unit must either have an TemporalUnit.isDurationEstimated() exact duration or
        be ChronoUnit.DAYS which is treated as 24 hours. Other units throw an exception.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - amountToAdd: the amount to add, measured in terms of the unit, positive or negative
        - unit: the unit that the amount is measured in, must have an exact duration, not null

        Returns
        - a `Duration` based on this duration with the specified duration added, not null

        Raises
        - UnsupportedTemporalTypeException: if the unit is not supported
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plusDays(self, daysToAdd: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in standard 24 hour days added.
        
        The number of days is multiplied by 86400 to obtain the number of seconds to add.
        This is based on the standard definition of a day as 24 hours.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - daysToAdd: the days to add, positive or negative

        Returns
        - a `Duration` based on this duration with the specified days added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plusHours(self, hoursToAdd: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in hours added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - hoursToAdd: the hours to add, positive or negative

        Returns
        - a `Duration` based on this duration with the specified hours added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plusMinutes(self, minutesToAdd: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in minutes added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - minutesToAdd: the minutes to add, positive or negative

        Returns
        - a `Duration` based on this duration with the specified minutes added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plusSeconds(self, secondsToAdd: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in seconds added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - secondsToAdd: the seconds to add, positive or negative

        Returns
        - a `Duration` based on this duration with the specified seconds added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plusMillis(self, millisToAdd: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in milliseconds added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - millisToAdd: the milliseconds to add, positive or negative

        Returns
        - a `Duration` based on this duration with the specified milliseconds added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def plusNanos(self, nanosToAdd: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in nanoseconds added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - nanosToAdd: the nanoseconds to add, positive or negative

        Returns
        - a `Duration` based on this duration with the specified nanoseconds added, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minus(self, duration: "Duration") -> "Duration":
        """
        Returns a copy of this duration with the specified duration subtracted.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - duration: the duration to subtract, positive or negative, not null

        Returns
        - a `Duration` based on this duration with the specified duration subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minus(self, amountToSubtract: int, unit: "TemporalUnit") -> "Duration":
        """
        Returns a copy of this duration with the specified duration subtracted.
        
        The duration amount is measured in terms of the specified unit.
        Only a subset of units are accepted by this method.
        The unit must either have an TemporalUnit.isDurationEstimated() exact duration or
        be ChronoUnit.DAYS which is treated as 24 hours. Other units throw an exception.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - amountToSubtract: the amount to subtract, measured in terms of the unit, positive or negative
        - unit: the unit that the amount is measured in, must have an exact duration, not null

        Returns
        - a `Duration` based on this duration with the specified duration subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minusDays(self, daysToSubtract: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in standard 24 hour days subtracted.
        
        The number of days is multiplied by 86400 to obtain the number of seconds to subtract.
        This is based on the standard definition of a day as 24 hours.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - daysToSubtract: the days to subtract, positive or negative

        Returns
        - a `Duration` based on this duration with the specified days subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minusHours(self, hoursToSubtract: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in hours subtracted.
        
        The number of hours is multiplied by 3600 to obtain the number of seconds to subtract.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - hoursToSubtract: the hours to subtract, positive or negative

        Returns
        - a `Duration` based on this duration with the specified hours subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minusMinutes(self, minutesToSubtract: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in minutes subtracted.
        
        The number of hours is multiplied by 60 to obtain the number of seconds to subtract.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - minutesToSubtract: the minutes to subtract, positive or negative

        Returns
        - a `Duration` based on this duration with the specified minutes subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minusSeconds(self, secondsToSubtract: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in seconds subtracted.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - secondsToSubtract: the seconds to subtract, positive or negative

        Returns
        - a `Duration` based on this duration with the specified seconds subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minusMillis(self, millisToSubtract: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in milliseconds subtracted.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - millisToSubtract: the milliseconds to subtract, positive or negative

        Returns
        - a `Duration` based on this duration with the specified milliseconds subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def minusNanos(self, nanosToSubtract: int) -> "Duration":
        """
        Returns a copy of this duration with the specified duration in nanoseconds subtracted.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - nanosToSubtract: the nanoseconds to subtract, positive or negative

        Returns
        - a `Duration` based on this duration with the specified nanoseconds subtracted, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def multipliedBy(self, multiplicand: int) -> "Duration":
        """
        Returns a copy of this duration multiplied by the scalar.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - multiplicand: the value to multiply the duration by, positive or negative

        Returns
        - a `Duration` based on this duration multiplied by the specified scalar, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def dividedBy(self, divisor: int) -> "Duration":
        """
        Returns a copy of this duration divided by the specified value.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - divisor: the value to divide the duration by, positive or negative, not zero

        Returns
        - a `Duration` based on this duration divided by the specified divisor, not null

        Raises
        - ArithmeticException: if the divisor is zero or if numeric overflow occurs
        """
        ...


    def dividedBy(self, divisor: "Duration") -> int:
        """
        Returns number of whole times a specified Duration occurs within this Duration.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - divisor: the value to divide the duration by, positive or negative, not null

        Returns
        - number of whole times, rounded toward zero, a specified
                `Duration` occurs within this Duration, may be negative

        Raises
        - ArithmeticException: if the divisor is zero, or if numeric overflow occurs

        Since
        - 9
        """
        ...


    def negated(self) -> "Duration":
        """
        Returns a copy of this duration with the length negated.
        
        This method swaps the sign of the total length of this duration.
        For example, `PT1.3S` will be returned as `PT-1.3S`.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - a `Duration` based on this duration with the amount negated, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def abs(self) -> "Duration":
        """
        Returns a copy of this duration with a positive length.
        
        This method returns a positive duration by effectively removing the sign from any negative total length.
        For example, `PT-1.3S` will be returned as `PT1.3S`.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - a `Duration` based on this duration with an absolute length, not null

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def addTo(self, temporal: "Temporal") -> "Temporal":
        """
        Adds this duration to the specified temporal object.
        
        This returns a temporal object of the same observable type as the input
        with this duration added.
        
        In most cases, it is clearer to reverse the calling pattern by using
        Temporal.plus(TemporalAmount).
        ```
          // these two lines are equivalent, but the second approach is recommended
          dateTime = thisDuration.addTo(dateTime);
          dateTime = dateTime.plus(thisDuration);
        ```
        
        The calculation will add the seconds, then nanos.
        Only non-zero amounts will be added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - temporal: the temporal object to adjust, not null

        Returns
        - an object of the same type with the adjustment made, not null

        Raises
        - DateTimeException: if unable to add
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def subtractFrom(self, temporal: "Temporal") -> "Temporal":
        """
        Subtracts this duration from the specified temporal object.
        
        This returns a temporal object of the same observable type as the input
        with this duration subtracted.
        
        In most cases, it is clearer to reverse the calling pattern by using
        Temporal.minus(TemporalAmount).
        ```
          // these two lines are equivalent, but the second approach is recommended
          dateTime = thisDuration.subtractFrom(dateTime);
          dateTime = dateTime.minus(thisDuration);
        ```
        
        The calculation will subtract the seconds, then nanos.
        Only non-zero amounts will be added.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - temporal: the temporal object to adjust, not null

        Returns
        - an object of the same type with the adjustment made, not null

        Raises
        - DateTimeException: if unable to subtract
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def toDays(self) -> int:
        """
        Gets the number of days in this duration.
        
        This returns the total number of days in the duration by dividing the
        number of seconds by 86400.
        This is based on the standard definition of a day as 24 hours.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of days in the duration, may be negative
        """
        ...


    def toHours(self) -> int:
        """
        Gets the number of hours in this duration.
        
        This returns the total number of hours in the duration by dividing the
        number of seconds by 3600.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of hours in the duration, may be negative
        """
        ...


    def toMinutes(self) -> int:
        """
        Gets the number of minutes in this duration.
        
        This returns the total number of minutes in the duration by dividing the
        number of seconds by 60.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of minutes in the duration, may be negative
        """
        ...


    def toSeconds(self) -> int:
        """
        Gets the number of seconds in this duration.
        
        This returns the total number of whole seconds in the duration.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the whole seconds part of the length of the duration, positive or negative

        Since
        - 9
        """
        ...


    def toMillis(self) -> int:
        """
        Converts this duration to the total length in milliseconds.
        
        If this duration is too large to fit in a `long` milliseconds, then an
        exception is thrown.
        
        If this duration has greater than millisecond precision, then the conversion
        will drop any excess precision information as though the amount in nanoseconds
        was subject to integer division by one million.

        Returns
        - the total length of the duration in milliseconds

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def toNanos(self) -> int:
        """
        Converts this duration to the total length in nanoseconds expressed as a `long`.
        
        If this duration is too large to fit in a `long` nanoseconds, then an
        exception is thrown.

        Returns
        - the total length of the duration in nanoseconds

        Raises
        - ArithmeticException: if numeric overflow occurs
        """
        ...


    def toDaysPart(self) -> int:
        """
        Extracts the number of days in the duration.
        
        This returns the total number of days in the duration by dividing the
        number of seconds by 86400.
        This is based on the standard definition of a day as 24 hours.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of days in the duration, may be negative

        Since
        - 9
        """
        ...


    def toHoursPart(self) -> int:
        """
        Extracts the number of hours part in the duration.
        
        This returns the number of remaining hours when dividing .toHours
        by hours in a day.
        This is based on the standard definition of a day as 24 hours.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of hours part in the duration, may be negative

        Since
        - 9
        """
        ...


    def toMinutesPart(self) -> int:
        """
        Extracts the number of minutes part in the duration.
        
        This returns the number of remaining minutes when dividing .toMinutes
        by minutes in an hour.
        This is based on the standard definition of an hour as 60 minutes.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of minutes parts in the duration, may be negative

        Since
        - 9
        """
        ...


    def toSecondsPart(self) -> int:
        """
        Extracts the number of seconds part in the duration.
        
        This returns the remaining seconds when dividing .toSeconds
        by seconds in a minute.
        This is based on the standard definition of a minute as 60 seconds.
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of seconds parts in the duration, may be negative

        Since
        - 9
        """
        ...


    def toMillisPart(self) -> int:
        """
        Extracts the number of milliseconds part of the duration.
        
        This returns the milliseconds part by dividing the number of nanoseconds by 1,000,000.
        The length of the duration is stored using two fields - seconds and nanoseconds.
        The nanoseconds part is a value from 0 to 999,999,999 that is an adjustment to
        the length in seconds.
        The total duration is defined by calling .getNano() and .getSeconds().
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the number of milliseconds part of the duration.

        Since
        - 9
        """
        ...


    def toNanosPart(self) -> int:
        """
        Get the nanoseconds part within seconds of the duration.
        
        The length of the duration is stored using two fields - seconds and nanoseconds.
        The nanoseconds part is a value from 0 to 999,999,999 that is an adjustment to
        the length in seconds.
        The total duration is defined by calling .getNano() and .getSeconds().
        
        This instance is immutable and unaffected by this method call.

        Returns
        - the nanoseconds within the second part of the length of the duration, from 0 to 999,999,999

        Since
        - 9
        """
        ...


    def truncatedTo(self, unit: "TemporalUnit") -> "Duration":
        """
        Returns a copy of this `Duration` truncated to the specified unit.
        
        Truncating the duration returns a copy of the original with conceptual fields
        smaller than the specified unit set to zero.
        For example, truncating with the ChronoUnit.MINUTES MINUTES unit will
        round down towards zero to the nearest minute, setting the seconds and
        nanoseconds to zero.
        
        The unit must have a TemporalUnit.getDuration() duration
        that divides into the length of a standard day without remainder.
        This includes all
        ChronoUnit.isTimeBased() time-based units on {@code ChronoUnit}
        and ChronoUnit.DAYS DAYS. Other ChronoUnits throw an exception.
        
        This instance is immutable and unaffected by this method call.

        Arguments
        - unit: the unit to truncate to, not null

        Returns
        - a `Duration` based on this duration with the time truncated, not null

        Raises
        - DateTimeException: if the unit is invalid for truncation
        - UnsupportedTemporalTypeException: if the unit is not supported

        Since
        - 9
        """
        ...


    def compareTo(self, otherDuration: "Duration") -> int:
        """
        Compares this duration to the specified `Duration`.
        
        The comparison is based on the total length of the durations.
        It is "consistent with equals", as defined by Comparable.

        Arguments
        - otherDuration: the other duration to compare to, not null

        Returns
        - the comparator value, negative if less, positive if greater
        """
        ...


    def equals(self, other: "Object") -> bool:
        """
        Checks if this duration is equal to the specified `Duration`.
        
        The comparison is based on the total length of the durations.

        Arguments
        - other: the other duration, null returns False

        Returns
        - True if the other duration is equal to this one
        """
        ...


    def hashCode(self) -> int:
        """
        A hash code for this duration.

        Returns
        - a suitable hash code
        """
        ...


    def toString(self) -> str:
        """
        A string representation of this duration using ISO-8601 seconds
        based representation, such as `PT8H6M12.345S`.
        
        The format of the returned string will be `PTnHnMnS`, where n is
        the relevant hours, minutes or seconds part of the duration.
        Any fractional seconds are placed after a decimal point in the seconds section.
        If a section has a zero value, it is omitted.
        The hours, minutes and seconds will all have the same sign.
        
        Examples:
        ```
           "20.345 seconds"                 -- "PT20.345S
           "15 minutes" (15 * 60 seconds)   -- "PT15M"
           "10 hours" (10 * 3600 seconds)   -- "PT10H"
           "2 days" (2 * 86400 seconds)     -- "PT48H"
        ```
        Note that multiples of 24 hours are not output as days to avoid confusion
        with `Period`.

        Returns
        - an ISO-8601 representation of this duration, not null
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.SmithingRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class SmithingRecipe(Recipe, Keyed):
    """
    Represents a smithing recipe.
    """

    def __init__(self, key: "NamespacedKey", result: "ItemStack", base: "RecipeChoice", addition: "RecipeChoice"):
        """
        Create a smithing recipe to produce the specified result ItemStack.

        Arguments
        - key: The unique recipe key
        - result: The item you want the recipe to create.
        - base: The base ingredient
        - addition: The addition ingredient

        Deprecated
        - as of Minecraft 1.20, smithing recipes are now separated into two
        distinct recipe types, SmithingTransformRecipe and SmithingTrimRecipe.
        This class now acts as a base class to these two classes and will do nothing when
        added to the server.
        """
        ...


    def getBase(self) -> "RecipeChoice":
        """
        Get the base recipe item.

        Returns
        - base choice
        """
        ...


    def getAddition(self) -> "RecipeChoice":
        """
        Get the addition recipe item.

        Returns
        - addition choice
        """
        ...


    def getResult(self) -> "ItemStack":
        ...


    def getKey(self) -> "NamespacedKey":
        ...



"""
Python module generated from Java source file org.bukkit.event.weather.LightningStrikeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import World
from org.bukkit.entity import LightningStrike
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.weather import *
from typing import Any, Callable, Iterable, Tuple


class LightningStrikeEvent(WeatherEvent, Cancellable):
    """
    Stores data for lightning striking
    """

    def __init__(self, world: "World", bolt: "LightningStrike"):
        ...


    def __init__(self, world: "World", bolt: "LightningStrike", cause: "Cause"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getLightning(self) -> "LightningStrike":
        """
        Gets the bolt which is striking the earth.

        Returns
        - lightning entity
        """
        ...


    def getCause(self) -> "Cause":
        """
        Gets the cause of this lightning strike.

        Returns
        - strike cause
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class Cause(Enum):

        COMMAND = 0
        """
        Triggered by the /summon command.
        """
        CUSTOM = 1
        """
        Triggered by a Plugin.
        """
        SPAWNER = 2
        """
        Triggered by a Spawner.
        """
        TRIDENT = 3
        """
        Triggered by an enchanted trident.
        """
        TRAP = 4
        """
        Triggered by a skeleton horse trap.
        """
        WEATHER = 5
        """
        Triggered by weather.
        """
        ENCHANTMENT = 6
        """
        Triggered by an enchantment but not a trident.
        """
        UNKNOWN = 7
        """
        Unknown trigger.
        """



"""
Python module generated from Java source file org.yaml.snakeyaml.introspector.BeanAccess

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.yaml.snakeyaml.introspector import *
from typing import Any, Callable, Iterable, Tuple


class BeanAccess(Enum):
    """
    Control instance variables.
    """

    DEFAULT = 0
    """
    use JavaBean properties and public fields
    """
    FIELD = 1
    """
    use all declared fields (including inherited)
    """
    PROPERTY = 2
    """
    reserved
    """



"""
Python module generated from Java source file org.bukkit.entity.PufferFish

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class PufferFish(Fish):
    """
    Represents a puffer fish.
    """

    def getPuffState(self) -> int:
        """
        Returns the current puff state of this fish (i.e. how inflated it is).

        Returns
        - current puff state
        """
        ...


    def setPuffState(self, state: int) -> None:
        """
        Sets the current puff state of this fish (i.e. how inflated it is).

        Arguments
        - state: new puff state
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ClassToInstanceMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ClassToInstanceMap(Map):
    """
    A map, each entry of which maps a Java <a href="http://tinyurl.com/2cmwkz">raw type</a> to an
    instance of that type. In addition to implementing `Map`, the additional type-safe
    operations .putInstance and .getInstance are available.
    
    Like any other `Map<Class, Object>`, this map may contain entries for primitive types,
    and a primitive type and its corresponding wrapper type may map to different values.
    
    <h3>Implementations</h3>
    
    
      - ImmutableClassToInstanceMap
      - MutableClassToInstanceMap
    
    
    To map a generic type to an instance of that type, use com.google.common.reflect.TypeToInstanceMap instead.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#classtoinstancemap">`ClassToInstanceMap`</a>.
    
    Type `<B>`: the common supertype that all values will share. When in doubt, just use Object, or use `@Nullable Object` to allow null values.

    Since
    - 2.0
    """

    def getInstance(self, type: type["T"]) -> "T":
        """
        Returns the value the specified class is mapped to, or `null` if no entry for this class
        is present. This will only return a value that was bound to this specific class, not a value
        that may have been bound to a subtype.
        """
        ...


    def putInstance(self, type: type["T"], value: "T") -> "T":
        """
        Maps the specified class to the specified value. Does *not* associate this value with any
        of the class's supertypes.

        Returns
        - the value previously associated with this class (possibly `null`), or `null` if there was no previous entry.
        """
        ...



"""
Python module generated from Java source file java.net.HttpURLConnection

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import InputStream
from java.net import *
from java.security import Permission
from java.util import Date
from typing import Any, Callable, Iterable, Tuple


class HttpURLConnection(URLConnection):
    """
    A URLConnection with support for HTTP-specific features. See
    <A HREF="http://www.w3.org/pub/WWW/Protocols/"> the spec </A> for
    details.
    
    
    Each HttpURLConnection instance is used to make a single request
    but the underlying network connection to the HTTP server may be
    transparently shared by other instances. Calling the close() methods
    on the InputStream or OutputStream of an HttpURLConnection
    after a request may free network resources associated with this
    instance but has no effect on any shared persistent connection.
    Calling the disconnect() method may close the underlying socket
    if a persistent connection is otherwise idle at that time.
    
    <P>The HTTP protocol handler has a few settings that can be accessed through
    System Properties. This covers
    <a href="doc-files/net-properties.html#Proxies">Proxy settings</a> as well as
    <a href="doc-files/net-properties.html#MiscHTTP"> various other settings</a>.
    </P>
    
    **Security permissions**
    
    If a security manager is installed, and if a method is called which results in an
    attempt to open a connection, the caller must possess either:
    - a "connect" SocketPermission to the host/port combination of the
    destination URL or
    - a URLPermission that permits this request.
    
    If automatic redirection is enabled, and this request is redirected to another
    destination, then the caller must also have permission to connect to the
    redirected host/URL.

    See
    - java.net.HttpURLConnection.disconnect()

    Since
    - 1.1
    """

    HTTP_OK = 200
    """
    HTTP Status-Code 200: OK.
    """
    HTTP_CREATED = 201
    """
    HTTP Status-Code 201: Created.
    """
    HTTP_ACCEPTED = 202
    """
    HTTP Status-Code 202: Accepted.
    """
    HTTP_NOT_AUTHORITATIVE = 203
    """
    HTTP Status-Code 203: Non-Authoritative Information.
    """
    HTTP_NO_CONTENT = 204
    """
    HTTP Status-Code 204: No Content.
    """
    HTTP_RESET = 205
    """
    HTTP Status-Code 205: Reset Content.
    """
    HTTP_PARTIAL = 206
    """
    HTTP Status-Code 206: Partial Content.
    """
    HTTP_MULT_CHOICE = 300
    """
    HTTP Status-Code 300: Multiple Choices.
    """
    HTTP_MOVED_PERM = 301
    """
    HTTP Status-Code 301: Moved Permanently.
    """
    HTTP_MOVED_TEMP = 302
    """
    HTTP Status-Code 302: Temporary Redirect.
    """
    HTTP_SEE_OTHER = 303
    """
    HTTP Status-Code 303: See Other.
    """
    HTTP_NOT_MODIFIED = 304
    """
    HTTP Status-Code 304: Not Modified.
    """
    HTTP_USE_PROXY = 305
    """
    HTTP Status-Code 305: Use Proxy.
    """
    HTTP_BAD_REQUEST = 400
    """
    HTTP Status-Code 400: Bad Request.
    """
    HTTP_UNAUTHORIZED = 401
    """
    HTTP Status-Code 401: Unauthorized.
    """
    HTTP_PAYMENT_REQUIRED = 402
    """
    HTTP Status-Code 402: Payment Required.
    """
    HTTP_FORBIDDEN = 403
    """
    HTTP Status-Code 403: Forbidden.
    """
    HTTP_NOT_FOUND = 404
    """
    HTTP Status-Code 404: Not Found.
    """
    HTTP_BAD_METHOD = 405
    """
    HTTP Status-Code 405: Method Not Allowed.
    """
    HTTP_NOT_ACCEPTABLE = 406
    """
    HTTP Status-Code 406: Not Acceptable.
    """
    HTTP_PROXY_AUTH = 407
    """
    HTTP Status-Code 407: Proxy Authentication Required.
    """
    HTTP_CLIENT_TIMEOUT = 408
    """
    HTTP Status-Code 408: Request Time-Out.
    """
    HTTP_CONFLICT = 409
    """
    HTTP Status-Code 409: Conflict.
    """
    HTTP_GONE = 410
    """
    HTTP Status-Code 410: Gone.
    """
    HTTP_LENGTH_REQUIRED = 411
    """
    HTTP Status-Code 411: Length Required.
    """
    HTTP_PRECON_FAILED = 412
    """
    HTTP Status-Code 412: Precondition Failed.
    """
    HTTP_ENTITY_TOO_LARGE = 413
    """
    HTTP Status-Code 413: Request Entity Too Large.
    """
    HTTP_REQ_TOO_LONG = 414
    """
    HTTP Status-Code 414: Request-URI Too Large.
    """
    HTTP_UNSUPPORTED_TYPE = 415
    """
    HTTP Status-Code 415: Unsupported Media Type.
    """
    HTTP_SERVER_ERROR = 500
    """
    HTTP Status-Code 500: Internal Server Error.

    Deprecated
    - it is misplaced and shouldn't have existed.
    """
    HTTP_INTERNAL_ERROR = 500
    """
    HTTP Status-Code 500: Internal Server Error.
    """
    HTTP_NOT_IMPLEMENTED = 501
    """
    HTTP Status-Code 501: Not Implemented.
    """
    HTTP_BAD_GATEWAY = 502
    """
    HTTP Status-Code 502: Bad Gateway.
    """
    HTTP_UNAVAILABLE = 503
    """
    HTTP Status-Code 503: Service Unavailable.
    """
    HTTP_GATEWAY_TIMEOUT = 504
    """
    HTTP Status-Code 504: Gateway Timeout.
    """
    HTTP_VERSION = 505
    """
    HTTP Status-Code 505: HTTP Version Not Supported.
    """


    def setAuthenticator(self, auth: "Authenticator") -> None:
        """
        Supplies an java.net.Authenticator Authenticator to be used
        when authentication is requested through the HTTP protocol for
        this `HttpURLConnection`.
        If no authenticator is supplied, the
        Authenticator.setDefault(java.net.Authenticator) default
        authenticator will be used.

        Arguments
        - auth: The `Authenticator` that should be used by this
                  `HttpURLConnection`.

        Raises
        - UnsupportedOperationException: if setting an Authenticator is
                 not supported by the underlying implementation.
        - IllegalStateException: if URLConnection is already connected.
        - NullPointerException: if the supplied `auth` is `null`.

        Since
        - 9

        Unknown Tags
        - The default behavior of this method is to unconditionally
                  throw UnsupportedOperationException. Concrete
                  implementations of `HttpURLConnection`
                  which support supplying an `Authenticator` for a
                  specific `HttpURLConnection` instance should
                  override this method to implement a different behavior.
        - Depending on authentication schemes, an implementation
                  may or may not need to use the provided authenticator
                  to obtain a password. For instance, an implementation that
                  relies on third-party security libraries may still invoke the
                  default authenticator if these libraries are configured
                  to do so.
                  Likewise, an implementation that supports transparent
                  NTLM authentication may let the system attempt
                  to connect using the system user credentials first,
                  before invoking the provided authenticator.
                  
                  However, if an authenticator is specifically provided,
                  then the underlying connection may only be reused for
                  `HttpURLConnection` instances which share the same
                  `Authenticator` instance, and authentication information,
                  if cached, may only be reused for an `HttpURLConnection`
                  sharing that same `Authenticator`.
        """
        ...


    def getHeaderFieldKey(self, n: int) -> str:
        """
        Returns the key for the `n`<sup>th</sup> header field.
        Some implementations may treat the `0`<sup>th</sup>
        header field as special, i.e. as the status line returned by the HTTP
        server. In this case, .getHeaderField(int) getHeaderField(0) returns the status
        line, but `getHeaderFieldKey(0)` returns null.

        Arguments
        - n: an index, where `n >=0`.

        Returns
        - the key for the `n`<sup>th</sup> header field,
                 or `null` if the key does not exist.
        """
        ...


    def setFixedLengthStreamingMode(self, contentLength: int) -> None:
        """
        This method is used to enable streaming of a HTTP request body
        without internal buffering, when the content length is known in
        advance.
        
        An exception will be thrown if the application
        attempts to write more data than the indicated
        content-length, or if the application closes the OutputStream
        before writing the indicated amount.
        
        When output streaming is enabled, authentication
        and redirection cannot be handled automatically.
        A HttpRetryException will be thrown when reading
        the response if authentication or redirection are required.
        This exception can be queried for the details of the error.
        
        This method must be called before the URLConnection is connected.
        
        <B>NOTE:</B> .setFixedLengthStreamingMode(long) is recommended
        instead of this method as it allows larger content lengths to be set.

        Arguments
        - contentLength: The number of bytes which will be written
                 to the OutputStream.

        Raises
        - IllegalStateException: if URLConnection is already connected
                 or if a different streaming mode is already enabled.
        - IllegalArgumentException: if a content length less than
                 zero is specified.

        See
        - .setChunkedStreamingMode(int)

        Since
        - 1.5
        """
        ...


    def setFixedLengthStreamingMode(self, contentLength: int) -> None:
        """
        This method is used to enable streaming of a HTTP request body
        without internal buffering, when the content length is known in
        advance.
        
        <P> An exception will be thrown if the application attempts to write
        more data than the indicated content-length, or if the application
        closes the OutputStream before writing the indicated amount.
        
        <P> When output streaming is enabled, authentication and redirection
        cannot be handled automatically. A HttpRetryException will
        be thrown when reading the response if authentication or redirection
        are required. This exception can be queried for the details of the
        error.
        
        <P> This method must be called before the URLConnection is connected.
        
        <P> The content length set by invoking this method takes precedence
        over any value set by .setFixedLengthStreamingMode(int).

        Arguments
        - contentLength: The number of bytes which will be written to the OutputStream.

        Raises
        - IllegalStateException: if URLConnection is already connected or if a different
                 streaming mode is already enabled.
        - IllegalArgumentException: if a content length less than zero is specified.

        Since
        - 1.7
        """
        ...


    def setChunkedStreamingMode(self, chunklen: int) -> None:
        """
        This method is used to enable streaming of a HTTP request body
        without internal buffering, when the content length is **not**
        known in advance. In this mode, chunked transfer encoding
        is used to send the request body. Note, not all HTTP servers
        support this mode.
        
        When output streaming is enabled, authentication
        and redirection cannot be handled automatically.
        A HttpRetryException will be thrown when reading
        the response if authentication or redirection are required.
        This exception can be queried for the details of the error.
        
        This method must be called before the URLConnection is connected.

        Arguments
        - chunklen: The number of bytes to write in each chunk.
                 If chunklen is less than or equal to zero, a default
                 value will be used.

        Raises
        - IllegalStateException: if URLConnection is already connected
                 or if a different streaming mode is already enabled.

        See
        - .setFixedLengthStreamingMode(int)

        Since
        - 1.5
        """
        ...


    def getHeaderField(self, n: int) -> str:
        """
        Returns the value for the `n`<sup>th</sup> header field.
        Some implementations may treat the `0`<sup>th</sup>
        header field as special, i.e. as the status line returned by the HTTP
        server.
        
        This method can be used in conjunction with the
        .getHeaderFieldKey getHeaderFieldKey method to iterate through all
        the headers in the message.

        Arguments
        - n: an index, where `n>=0`.

        Returns
        - the value of the `n`<sup>th</sup> header field,
                 or `null` if the value does not exist.

        See
        - java.net.HttpURLConnection.getHeaderFieldKey(int)
        """
        ...


    @staticmethod
    def setFollowRedirects(set: bool) -> None:
        """
        Sets whether HTTP redirects  (requests with response code 3xx) should
        be automatically followed by this class.  True by default.  Applets
        cannot change this variable.
        
        If there is a security manager, this method first calls
        the security manager's `checkSetFactory` method
        to ensure the operation is allowed.
        This could result in a SecurityException.

        Arguments
        - set: a `boolean` indicating whether or not
        to follow HTTP redirects.

        Raises
        - SecurityException: if a security manager exists and its
                    `checkSetFactory` method doesn't
                    allow the operation.

        See
        - .getFollowRedirects()
        """
        ...


    @staticmethod
    def getFollowRedirects() -> bool:
        """
        Returns a `boolean` indicating
        whether or not HTTP redirects (3xx) should
        be automatically followed.

        Returns
        - `True` if HTTP redirects should
        be automatically followed, `False` if not.

        See
        - .setFollowRedirects(boolean)
        """
        ...


    def setInstanceFollowRedirects(self, followRedirects: bool) -> None:
        """
        Sets whether HTTP redirects (requests with response code 3xx) should
        be automatically followed by this `HttpURLConnection`
        instance.
        
        The default value comes from followRedirects, which defaults to
        True.

        Arguments
        - followRedirects: a `boolean` indicating
        whether or not to follow HTTP redirects.

        See
        - .getInstanceFollowRedirects

        Since
        - 1.3
        """
        ...


    def getInstanceFollowRedirects(self) -> bool:
        """
        Returns the value of this `HttpURLConnection`'s
        `instanceFollowRedirects` field.

        Returns
        - the value of this `HttpURLConnection`'s
                 `instanceFollowRedirects` field.

        See
        - .setInstanceFollowRedirects(boolean)

        Since
        - 1.3
        """
        ...


    def setRequestMethod(self, method: str) -> None:
        """
        Set the method for the URL request, one of:
        <UL>
         <LI>GET
         <LI>POST
         <LI>HEAD
         <LI>OPTIONS
         <LI>PUT
         <LI>DELETE
         <LI>TRACE
        </UL> are legal, subject to protocol restrictions.  The default
        method is GET.

        Arguments
        - method: the HTTP method

        Raises
        - ProtocolException: if the method cannot be reset or if
                     the requested method isn't valid for HTTP.
        - SecurityException: if a security manager is set and the
                     method is "TRACE", but the "allowHttpTrace"
                     NetPermission is not granted.

        See
        - .getRequestMethod()
        """
        ...


    def getRequestMethod(self) -> str:
        """
        Get the request method.

        Returns
        - the HTTP request method

        See
        - .setRequestMethod(java.lang.String)
        """
        ...


    def getResponseCode(self) -> int:
        """
        Gets the status code from an HTTP response message.
        For example, in the case of the following status lines:
        <PRE>
        HTTP/1.0 200 OK
        HTTP/1.0 401 Unauthorized
        </PRE>
        It will return 200 and 401 respectively.
        Returns -1 if no code can be discerned
        from the response (i.e., the response is not valid HTTP).

        Returns
        - the HTTP Status-Code, or -1

        Raises
        - IOException: if an error occurred connecting to the server.
        """
        ...


    def getResponseMessage(self) -> str:
        """
        Gets the HTTP response message, if any, returned along with the
        response code from a server.  From responses like:
        <PRE>
        HTTP/1.0 200 OK
        HTTP/1.0 404 Not Found
        </PRE>
        Extracts the Strings "OK" and "Not Found" respectively.
        Returns null if none could be discerned from the responses
        (the result was not valid HTTP).

        Returns
        - the HTTP response message, or `null`

        Raises
        - IOException: if an error occurred connecting to the server.
        """
        ...


    def getHeaderFieldDate(self, name: str, Default: int) -> int:
        ...


    def disconnect(self) -> None:
        """
        Indicates that other requests to the server
        are unlikely in the near future. Calling disconnect()
        should not imply that this HttpURLConnection
        instance can be reused for other requests.
        """
        ...


    def usingProxy(self) -> bool:
        """
        Indicates if the connection is going through a proxy.
        
        This method returns `True` if the connection is known
        to be going or has gone through proxies, and returns `False`
        if the connection will never go through a proxy or if
        the use of a proxy cannot be determined.

        Returns
        - a boolean indicating if the connection is using a proxy.
        """
        ...


    def getPermission(self) -> "Permission":
        """
        Returns a SocketPermission object representing the
        permission necessary to connect to the destination host and port.

        Returns
        - a `SocketPermission` object representing the
                permission necessary to connect to the destination
                host and port.

        Raises
        - IOException: if an error occurs while computing
                   the permission.
        """
        ...


    def getErrorStream(self) -> "InputStream":
        """
        Returns the error stream if the connection failed
        but the server sent useful data nonetheless. The
        typical example is when an HTTP server responds
        with a 404, which will cause a FileNotFoundException
        to be thrown in connect, but the server sent an HTML
        help page with suggestions as to what to do.
        
        This method will not cause a connection to be initiated.  If
        the connection was not connected, or if the server did not have
        an error while connecting or if the server had an error but
        no error data was sent, this method will return null. This is
        the default.

        Returns
        - an error stream if any, null if there have been no
        errors, the connection is not connected or the server sent no
        useful data.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.world.StructureGrowEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit import TreeType
from org.bukkit.block import BlockState
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.world import *
from typing import Any, Callable, Iterable, Tuple


class StructureGrowEvent(WorldEvent, Cancellable):
    """
    Event that is called when an organic structure attempts to grow (Sapling ->
    Tree), (Mushroom -> Huge Mushroom), naturally or using bonemeal.
    """

    def __init__(self, location: "Location", species: "TreeType", bonemeal: bool, player: "Player", blocks: list["BlockState"]):
        ...


    def getLocation(self) -> "Location":
        """
        Gets the location of the structure.

        Returns
        - Location of the structure
        """
        ...


    def getSpecies(self) -> "TreeType":
        """
        Gets the species type (birch, normal, pine, red mushroom, brown
        mushroom)

        Returns
        - Structure species
        """
        ...


    def isFromBonemeal(self) -> bool:
        """
        Checks if structure was grown using bonemeal.

        Returns
        - True if the structure was grown using bonemeal.
        """
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the player that created the structure.

        Returns
        - Player that created the structure, null if was not created
            manually
        """
        ...


    def getBlocks(self) -> list["BlockState"]:
        """
        Gets a list of all blocks associated with the structure.

        Returns
        - list of all blocks associated with the structure.
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractSetMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util import Collections
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractSetMultimap(AbstractMapBasedMultimap, SetMultimap):
    """
    Basic implementation of the SetMultimap interface. It's a wrapper around AbstractMapBasedMultimap that converts the returned collections into `Sets`. The .createCollection method must return a `Set`.

    Author(s)
    - Jared Levy
    """

    def get(self, key: "K") -> set["V"]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the Collection specified in the Multimap interface.
        """
        ...


    def entries(self) -> set["Entry"["K", "V"]]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the Collection specified in the Multimap interface.
        """
        ...


    def removeAll(self, key: "Object") -> set["V"]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the Collection specified in the Multimap interface.
        """
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> set["V"]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the Collection specified in the Multimap interface.
        
        Any duplicates in `values` will be stored in the multimap once.
        """
        ...


    def asMap(self) -> dict["K", Iterable["V"]]:
        """
        
        
        Though the method signature doesn't say so explicitly, the returned map has Set
        values.
        """
        ...


    def put(self, key: "K", value: "V") -> bool:
        """
        Stores a key-value pair in the multimap.

        Arguments
        - key: key to store in the multimap
        - value: value to store in the multimap

        Returns
        - `True` if the method increased the size of the multimap, or `False` if the
            multimap already contained the key-value pair
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Compares the specified object to this multimap for equality.
        
        Two `SetMultimap` instances are equal if, for each key, they contain the same values.
        Equality does not depend on the ordering of keys or values.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ImmutableTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import MoreObjects
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from com.google.errorprone.annotations import DoNotMock
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.util import Comparator
from java.util import Iterator
from java.util import Spliterator
from java.util.function import BinaryOperator
from java.util.function import Function
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ImmutableTable(AbstractTable, Serializable):
    """
    A Table whose contents will never change, with many other important properties detailed
    at ImmutableCollection.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/ImmutableCollectionsExplained">immutable collections</a>.

    Author(s)
    - Gregory Kick

    Since
    - 11.0
    """

    @staticmethod
    def toImmutableTable(rowFunction: "Function"["T", "R"], columnFunction: "Function"["T", "C"], valueFunction: "Function"["T", "V"]) -> "Collector"["T", Any, "ImmutableTable"["R", "C", "V"]]:
        """
        Returns a `Collector` that accumulates elements into an `ImmutableTable`. Each
        input element is mapped to one cell in the returned table, with the rows, columns, and values
        generated by applying the specified functions.
        
        The returned `Collector` will throw a `NullPointerException` at collection time
        if the row, column, or value functions return null on any input.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def toImmutableTable(rowFunction: "Function"["T", "R"], columnFunction: "Function"["T", "C"], valueFunction: "Function"["T", "V"], mergeFunction: "BinaryOperator"["V"]) -> "Collector"["T", Any, "ImmutableTable"["R", "C", "V"]]:
        """
        Returns a `Collector` that accumulates elements into an `ImmutableTable`. Each
        input element is mapped to one cell in the returned table, with the rows, columns, and values
        generated by applying the specified functions. If multiple inputs are mapped to the same row
        and column pair, they will be combined with the specified merging function in encounter order.
        
        The returned `Collector` will throw a `NullPointerException` at collection time
        if the row, column, value, or merging functions return null on any input.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def of() -> "ImmutableTable"["R", "C", "V"]:
        """
        Returns an empty immutable table.
        
        **Performance note:** the instance returned is a singleton.
        """
        ...


    @staticmethod
    def of(rowKey: "R", columnKey: "C", value: "V") -> "ImmutableTable"["R", "C", "V"]:
        """
        Returns an immutable table containing a single cell.
        """
        ...


    @staticmethod
    def copyOf(table: "Table"["R", "C", "V"]) -> "ImmutableTable"["R", "C", "V"]:
        """
        Returns an immutable copy of the provided table.
        
        The Table.cellSet() iteration order of the provided table determines the iteration
        ordering of all views in the returned table. Note that some views of the original table and the
        copied table may have different iteration orders. For more control over the ordering, create a
        Builder and call Builder.orderRowsBy, Builder.orderColumnsBy, and
        Builder.putAll
        
        Despite the method name, this method attempts to avoid actually copying the data when it is
        safe to do so. The exact circumstances under which a copy will or will not be performed are
        undocumented and subject to change.
        """
        ...


    @staticmethod
    def builder() -> "Builder"["R", "C", "V"]:
        """
        Returns a new builder. The generated builder is equivalent to the builder created by the Builder.Builder() ImmutableTable.Builder() constructor.
        """
        ...


    def cellSet(self) -> "ImmutableSet"["Cell"["R", "C", "V"]]:
        ...


    def values(self) -> "ImmutableCollection"["V"]:
        ...


    def column(self, columnKey: "C") -> "ImmutableMap"["R", "V"]:
        """
        Raises
        - NullPointerException: if `columnKey` is `null`
        """
        ...


    def columnKeySet(self) -> "ImmutableSet"["C"]:
        ...


    def columnMap(self) -> "ImmutableMap"["C", dict["R", "V"]]:
        """
        
        
        The value `Map<R, V>` instances in the returned map are ImmutableMap instances
        as well.
        """
        ...


    def row(self, rowKey: "R") -> "ImmutableMap"["C", "V"]:
        """
        Raises
        - NullPointerException: if `rowKey` is `null`
        """
        ...


    def rowKeySet(self) -> "ImmutableSet"["R"]:
        ...


    def rowMap(self) -> "ImmutableMap"["R", dict["C", "V"]]:
        """
        
        
        The value `Map<C, V>` instances in the returned map are ImmutableMap instances
        as well.
        """
        ...


    def contains(self, rowKey: "Object", columnKey: "Object") -> bool:
        ...


    def containsValue(self, value: "Object") -> bool:
        ...


    def clear(self) -> None:
        """
        Guaranteed to throw an exception and leave the table unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def put(self, rowKey: "R", columnKey: "C", value: "V") -> "V":
        """
        Guaranteed to throw an exception and leave the table unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def putAll(self, table: "Table"["R", "C", "V"]) -> None:
        """
        Guaranteed to throw an exception and leave the table unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def remove(self, rowKey: "Object", columnKey: "Object") -> "V":
        """
        Guaranteed to throw an exception and leave the table unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    class Builder:
        """
        A builder for creating immutable table instances, especially `public static final` tables
        ("constant tables"). Example:
        
        ````static final ImmutableTable<Integer, Character, String> SPREADSHEET =
            new ImmutableTable.Builder<Integer, Character, String>()
                .put(1, 'A', "foo")
                .put(1, 'B', "bar")
                .put(2, 'A', "baz")
                .buildOrThrow();````
        
        By default, the order in which cells are added to the builder determines the iteration
        ordering of all views in the returned table, with .putAll following the Table.cellSet() iteration order. However, if .orderRowsBy or .orderColumnsBy
        is called, the views are sorted by the supplied comparators.
        
        For empty or single-cell immutable tables, .of() and .of(Object, Object,
        Object) are even more convenient.
        
        Builder instances can be reused - it is safe to call .buildOrThrow multiple times to
        build multiple tables in series. Each table is a superset of the tables created before it.

        Since
        - 11.0
        """

        def __init__(self):
            """
            Creates a new builder. The returned builder is equivalent to the builder generated by ImmutableTable.builder.
            """
            ...


        def orderRowsBy(self, rowComparator: "Comparator"["R"]) -> "Builder"["R", "C", "V"]:
            """
            Specifies the ordering of the generated table's rows.
            """
            ...


        def orderColumnsBy(self, columnComparator: "Comparator"["C"]) -> "Builder"["R", "C", "V"]:
            """
            Specifies the ordering of the generated table's columns.
            """
            ...


        def put(self, rowKey: "R", columnKey: "C", value: "V") -> "Builder"["R", "C", "V"]:
            """
            Associates the (`rowKey`, `columnKey`) pair with `value` in the built
            table. Duplicate key pairs are not allowed and will cause .build to fail.
            """
            ...


        def put(self, cell: "Cell"["R", "C", "V"]) -> "Builder"["R", "C", "V"]:
            """
            Adds the given `cell` to the table, making it immutable if necessary. Duplicate key
            pairs are not allowed and will cause .build to fail.
            """
            ...


        def putAll(self, table: "Table"["R", "C", "V"]) -> "Builder"["R", "C", "V"]:
            """
            Associates all of the given table's keys and values in the built table. Duplicate row key
            column key pairs are not allowed, and will cause .build to fail.

            Raises
            - NullPointerException: if any key or value in `table` is null
            """
            ...


        def build(self) -> "ImmutableTable"["R", "C", "V"]:
            """
            Returns a newly-created immutable table.
            
            Prefer the equivalent method .buildOrThrow() to make it explicit that the method
            will throw an exception if there are duplicate key pairs. The `build()` method will
            soon be deprecated.

            Raises
            - IllegalArgumentException: if duplicate key pairs were added
            """
            ...


        def buildOrThrow(self) -> "ImmutableTable"["R", "C", "V"]:
            """
            Returns a newly-created immutable table, or throws an exception if duplicate key pairs were
            added.

            Raises
            - IllegalArgumentException: if duplicate key pairs were added

            Since
            - 31.0
            """
            ...



"""
Python module generated from Java source file com.google.common.graph.AbstractGraph

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.graph import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class AbstractGraph(AbstractBaseGraph, Graph):
    """
    This class provides a skeletal implementation of Graph. It is recommended to extend this
    class rather than implement Graph directly.
    
    Type `<N>`: Node parameter type

    Author(s)
    - James Sexton

    Since
    - 20.0
    """

    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this graph.
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.DirectExecutorService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations.concurrent import GuardedBy
from java.util import Collections
from java.util.concurrent import RejectedExecutionException
from java.util.concurrent import TimeUnit
from typing import Any, Callable, Iterable, Tuple


class DirectExecutorService(AbstractListeningExecutorService):
    """
    See newDirectExecutorService javadoc for behavioral notes.
    """

    def execute(self, command: "Runnable") -> None:
        ...


    def isShutdown(self) -> bool:
        ...


    def shutdown(self) -> None:
        ...


    def shutdownNow(self) -> list["Runnable"]:
        ...


    def isTerminated(self) -> bool:
        ...


    def awaitTermination(self, timeout: int, unit: "TimeUnit") -> bool:
        ...



"""
Python module generated from Java source file org.bukkit.Color

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableMap
from java.util import Arrays
from java.util import Locale
from java.util import Objects
from org.bukkit import *
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.configuration.serialization import SerializableAs
from typing import Any, Callable, Iterable, Tuple


class Color(ConfigurationSerializable):
    """
    A container for a color palette. This class is immutable; the set methods
    return a new color. The color names listed as fields are HTML4 standards,
    but subject to change.
    """

    WHITE = fromRGB(0xFFFFFF)
    """
    White, or (0xFF,0xFF,0xFF) in (R,G,B)
    """
    SILVER = fromRGB(0xC0C0C0)
    """
    Silver, or (0xC0,0xC0,0xC0) in (R,G,B)
    """
    GRAY = fromRGB(0x808080)
    """
    Gray, or (0x80,0x80,0x80) in (R,G,B)
    """
    BLACK = fromRGB(0x000000)
    """
    Black, or (0x00,0x00,0x00) in (R,G,B)
    """
    RED = fromRGB(0xFF0000)
    """
    Red, or (0xFF,0x00,0x00) in (R,G,B)
    """
    MAROON = fromRGB(0x800000)
    """
    Maroon, or (0x80,0x00,0x00) in (R,G,B)
    """
    YELLOW = fromRGB(0xFFFF00)
    """
    Yellow, or (0xFF,0xFF,0x00) in (R,G,B)
    """
    OLIVE = fromRGB(0x808000)
    """
    Olive, or (0x80,0x80,0x00) in (R,G,B)
    """
    LIME = fromRGB(0x00FF00)
    """
    Lime, or (0x00,0xFF,0x00) in (R,G,B)
    """
    GREEN = fromRGB(0x008000)
    """
    Green, or (0x00,0x80,0x00) in (R,G,B)
    """
    AQUA = fromRGB(0x00FFFF)
    """
    Aqua, or (0x00,0xFF,0xFF) in (R,G,B)
    """
    TEAL = fromRGB(0x008080)
    """
    Teal, or (0x00,0x80,0x80) in (R,G,B)
    """
    BLUE = fromRGB(0x0000FF)
    """
    Blue, or (0x00,0x00,0xFF) in (R,G,B)
    """
    NAVY = fromRGB(0x000080)
    """
    Navy, or (0x00,0x00,0x80) in (R,G,B)
    """
    FUCHSIA = fromRGB(0xFF00FF)
    """
    Fuchsia, or (0xFF,0x00,0xFF) in (R,G,B)
    """
    PURPLE = fromRGB(0x800080)
    """
    Purple, or (0x80,0x00,0x80) in (R,G,B)
    """
    ORANGE = fromRGB(0xFFA500)
    """
    Orange, or (0xFF,0xA5,0x00) in (R,G,B)
    """


    @staticmethod
    def fromARGB(alpha: int, red: int, green: int, blue: int) -> "Color":
        """
        Creates a new Color object from an alpha, red, green, and blue

        Arguments
        - alpha: integer from 0-255
        - red: integer from 0-255
        - green: integer from 0-255
        - blue: integer from 0-255

        Returns
        - a new Color object for the alpha, red, green, blue

        Raises
        - IllegalArgumentException: if any value is strictly >255 or <0
        """
        ...


    @staticmethod
    def fromRGB(red: int, green: int, blue: int) -> "Color":
        """
        Creates a new Color object from a red, green, and blue

        Arguments
        - red: integer from 0-255
        - green: integer from 0-255
        - blue: integer from 0-255

        Returns
        - a new Color object for the red, green, blue

        Raises
        - IllegalArgumentException: if any value is strictly >255 or <0
        """
        ...


    @staticmethod
    def fromBGR(blue: int, green: int, red: int) -> "Color":
        """
        Creates a new Color object from a blue, green, and red

        Arguments
        - blue: integer from 0-255
        - green: integer from 0-255
        - red: integer from 0-255

        Returns
        - a new Color object for the red, green, blue

        Raises
        - IllegalArgumentException: if any value is strictly >255 or <0
        """
        ...


    @staticmethod
    def fromRGB(rgb: int) -> "Color":
        """
        Creates a new color object from an integer that contains the red,
        green, and blue bytes in the lowest order 24 bits.

        Arguments
        - rgb: the integer storing the red, green, and blue values

        Returns
        - a new color object for specified values

        Raises
        - IllegalArgumentException: if any data is in the highest order 8
            bits
        """
        ...


    @staticmethod
    def fromARGB(argb: int) -> "Color":
        """
        Creates a new color object from an integer that contains the alpha, red,
        green, and blue bytes.

        Arguments
        - argb: the integer storing the alpha, red, green, and blue values

        Returns
        - a new color object for specified values
        """
        ...


    @staticmethod
    def fromBGR(bgr: int) -> "Color":
        """
        Creates a new color object from an integer that contains the blue,
        green, and red bytes in the lowest order 24 bits.

        Arguments
        - bgr: the integer storing the blue, green, and red values

        Returns
        - a new color object for specified values

        Raises
        - IllegalArgumentException: if any data is in the highest order 8
            bits
        """
        ...


    def getAlpha(self) -> int:
        """
        Gets the alpha component

        Returns
        - alpha component, from 0 to 255
        """
        ...


    def setAlpha(self, alpha: int) -> "Color":
        """
        Creates a new Color object with specified component

        Arguments
        - alpha: the alpha component, from 0 to 255

        Returns
        - a new color object with the red component
        """
        ...


    def getRed(self) -> int:
        """
        Gets the red component

        Returns
        - red component, from 0 to 255
        """
        ...


    def setRed(self, red: int) -> "Color":
        """
        Creates a new Color object with specified component

        Arguments
        - red: the red component, from 0 to 255

        Returns
        - a new color object with the red component
        """
        ...


    def getGreen(self) -> int:
        """
        Gets the green component

        Returns
        - green component, from 0 to 255
        """
        ...


    def setGreen(self, green: int) -> "Color":
        """
        Creates a new Color object with specified component

        Arguments
        - green: the red component, from 0 to 255

        Returns
        - a new color object with the red component
        """
        ...


    def getBlue(self) -> int:
        """
        Gets the blue component

        Returns
        - blue component, from 0 to 255
        """
        ...


    def setBlue(self, blue: int) -> "Color":
        """
        Creates a new Color object with specified component

        Arguments
        - blue: the red component, from 0 to 255

        Returns
        - a new color object with the red component
        """
        ...


    def asRGB(self) -> int:
        """
        Gets the color as an RGB integer.

        Returns
        - An integer representation of this color, as 0xRRGGBB
        """
        ...


    def asARGB(self) -> int:
        """
        Gets the color as an ARGB integer.

        Returns
        - An integer representation of this color, as 0xAARRGGBB
        """
        ...


    def asBGR(self) -> int:
        """
        Gets the color as an BGR integer.

        Returns
        - An integer representation of this color, as 0xBBGGRR
        """
        ...


    def mixDyes(self, *colors: Tuple["DyeColor", ...]) -> "Color":
        ...


    def mixColors(self, *colors: Tuple["Color", ...]) -> "Color":
        ...


    def equals(self, o: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def serialize(self) -> dict[str, "Object"]:
        ...


    @staticmethod
    def deserialize(map: dict[str, "Object"]) -> "Color":
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractSortedSetMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util import Collections
from java.util import NavigableSet
from java.util import SortedSet
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractSortedSetMultimap(AbstractSetMultimap, SortedSetMultimap):
    """
    Basic implementation of the SortedSetMultimap interface. It's a wrapper around AbstractMapBasedMultimap that converts the returned collections into sorted sets. The .createCollection method must return a `SortedSet`.

    Author(s)
    - Jared Levy
    """

    def get(self, key: "K") -> "SortedSet"["V"]:
        """
        Returns a collection view of all values associated with a key. If no mappings in the multimap
        have the provided key, an empty collection is returned.
        
        Changes to the returned collection will update the underlying multimap, and vice versa.
        
        Because a `SortedSetMultimap` has unique sorted values for a given key, this method
        returns a SortedSet, instead of the Collection specified in the Multimap interface.
        """
        ...


    def removeAll(self, key: "Object") -> "SortedSet"["V"]:
        """
        Removes all values associated with a given key. The returned collection is immutable.
        
        Because a `SortedSetMultimap` has unique sorted values for a given key, this method
        returns a SortedSet, instead of the Collection specified in the Multimap interface.
        """
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> "SortedSet"["V"]:
        """
        Stores a collection of values with the same key, replacing any existing values for that key.
        The returned collection is immutable.
        
        Because a `SortedSetMultimap` has unique sorted values for a given key, this method
        returns a SortedSet, instead of the Collection specified in the Multimap interface.
        
        Any duplicates in `values` will be stored in the multimap once.
        """
        ...


    def asMap(self) -> dict["K", Iterable["V"]]:
        """
        Returns a map view that associates each key with the corresponding values in the multimap.
        Changes to the returned map, such as element removal, will update the underlying multimap. The
        map does not support `setValue` on its entries, `put`, or `putAll`.
        
        When passed a key that is present in the map, `asMap().get(Object)` has the same
        behavior as .get, returning a live collection. When passed a key that is not present,
        however, `asMap().get(Object)` returns `null` instead of an empty collection.
        
        Though the method signature doesn't say so explicitly, the returned map has SortedSet values.
        """
        ...


    def values(self) -> Iterable["V"]:
        """
        
        
        Consequently, the values do not follow their natural ordering or the ordering of the value
        comparator.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.OminousItemSpawner

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class OminousItemSpawner(Entity):
    """
    Represents an ominous item spawner.
    """

    def getItem(self) -> "ItemStack":
        """
        Gets the item which will be spawned by this spawner.

        Returns
        - the item
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Sets the item which will be spawned by this spawner.

        Arguments
        - item: the item
        """
        ...


    def getSpawnItemAfterTicks(self) -> int:
        """
        Gets the ticks after which this item will be spawned.

        Returns
        - total spawn ticks
        """
        ...


    def setSpawnItemAfterTicks(self, ticks: int) -> None:
        """
        Sets the ticks after which this item will be spawned.

        Arguments
        - ticks: total spawn ticks
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.RedstoneWire

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block import BlockFace
from org.bukkit.block.data import AnaloguePowerable
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class RedstoneWire(AnaloguePowerable):
    """
    'north', 'east', 'south', 'west' represent the types of connections this
    redstone wire has to adjacent blocks.
    """

    def getFace(self, face: "BlockFace") -> "Connection":
        """
        Checks the type of connection on the specified face.

        Arguments
        - face: to check

        Returns
        - connection type
        """
        ...


    def setFace(self, face: "BlockFace", connection: "Connection") -> None:
        """
        Sets the type of connection on the specified face.

        Arguments
        - face: to set
        - connection: the connection type
        """
        ...


    def getAllowedFaces(self) -> set["BlockFace"]:
        """
        Gets all of this faces which may be set on this block.

        Returns
        - all allowed faces
        """
        ...


    class Connection(Enum):
        """
        The way in which a redstone wire can connect to an adjacent block face.
        """

        UP = 0
        """
        The wire travels up the side of the block adjacent to this face.
        """
        SIDE = 1
        """
        The wire travels flat from this face and into the adjacent block.
        """
        NONE = 2
        """
        The wire does not connect in this direction.
        """



"""
Python module generated from Java source file com.google.gson.stream.JsonScope

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.stream import *
from typing import Any, Callable, Iterable, Tuple


class JsonScope:
    """
    Lexical scoping elements within a JSON reader or writer.

    Author(s)
    - Jesse Wilson

    Since
    - 1.6
    """





"""
Python module generated from Java source file org.bukkit.entity.ThrownPotion

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from org.bukkit.potion import PotionEffect
from typing import Any, Callable, Iterable, Tuple


class ThrownPotion(ThrowableProjectile):
    """
    Represents a thrown potion bottle
    """

    def getEffects(self) -> Iterable["PotionEffect"]:
        """
        Returns the effects that are applied by this potion.

        Returns
        - The potion effects
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Returns a copy of the ItemStack for this thrown potion.
        
        Altering this copy will not alter the thrown potion directly. If you want
        to alter the thrown potion, you must use the .setItem(ItemStack) setItemStack method.

        Returns
        - A copy of the ItemStack for this thrown potion.
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Set the ItemStack for this thrown potion.
        
        The ItemStack must be of type org.bukkit.Material.SPLASH_POTION
        or org.bukkit.Material.LINGERING_POTION, otherwise an exception
        is thrown.

        Arguments
        - item: New ItemStack
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.NetworkBuilder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.base import Optional
from com.google.common.graph import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from typing import Any, Callable, Iterable, Tuple


class NetworkBuilder(AbstractGraphBuilder):
    """
    A builder for constructing instances of MutableNetwork or ImmutableNetwork with
    user-defined properties.
    
    A `Network` built by this class has the following default properties:
    
    
      - does not allow parallel edges
      - does not allow self-loops
      - orders Network.nodes() and Network.edges() in the order in which the
          elements were added (insertion order)
    
    
    `Network`s built by this class also guarantee that each collection-returning accessor
    returns a **(live) unmodifiable view**; see <a
    href="https://github.com/google/guava/wiki/GraphsExplained#accessor-behavior">the external
    documentation</a> for details.
    
    Examples of use:
    
    ````// Building a mutable network
    MutableNetwork<String, Integer> network =
        NetworkBuilder.directed().allowsParallelEdges(True).build();
    flightNetwork.addEdge("LAX", "ATL", 3025);
    flightNetwork.addEdge("LAX", "ATL", 1598);
    flightNetwork.addEdge("ATL", "LAX", 2450);
    
    // Building a immutable network
    ImmutableNetwork<String, Integer> immutableNetwork =
        NetworkBuilder.directed()
            .allowsParallelEdges(True)
            .<String, Integer>immutable()
            .addEdge("LAX", "ATL", 3025)
            .addEdge("LAX", "ATL", 1598)
            .addEdge("ATL", "LAX", 2450)
            .build();````
    
    Type `<N>`: The most general node type this builder will support. This is normally `Object`
        unless it is constrained by using a method like .nodeOrder, or the builder is
        constructed based on an existing `Network` using .from(Network).
    
    Type `<E>`: The most general edge type this builder will support. This is normally `Object`
        unless it is constrained by using a method like .edgeOrder, or the builder is
        constructed based on an existing `Network` using .from(Network).

    Author(s)
    - Joshua O'Madadhain

    Since
    - 20.0
    """

    @staticmethod
    def directed() -> "NetworkBuilder"["Object", "Object"]:
        """
        Returns a NetworkBuilder for building directed networks.
        """
        ...


    @staticmethod
    def undirected() -> "NetworkBuilder"["Object", "Object"]:
        """
        Returns a NetworkBuilder for building undirected networks.
        """
        ...


    @staticmethod
    def from(network: "Network"["N", "E"]) -> "NetworkBuilder"["N", "E"]:
        """
        Returns a NetworkBuilder initialized with all properties queryable from `network`.
        
        The "queryable" properties are those that are exposed through the Network interface,
        such as Network.isDirected(). Other properties, such as .expectedNodeCount(int), are not set in the new builder.
        """
        ...


    def immutable(self) -> "ImmutableNetwork.Builder"["N1", "E1"]:
        """
        Returns an ImmutableNetwork.Builder with the properties of this NetworkBuilder.
        
        The returned builder can be used for populating an ImmutableNetwork.

        Since
        - 28.0
        """
        ...


    def allowsParallelEdges(self, allowsParallelEdges: bool) -> "NetworkBuilder"["N", "E"]:
        """
        Specifies whether the network will allow parallel edges. Attempting to add a parallel edge to a
        network that does not allow them will throw an UnsupportedOperationException.
        
        The default value is `False`.
        """
        ...


    def allowsSelfLoops(self, allowsSelfLoops: bool) -> "NetworkBuilder"["N", "E"]:
        """
        Specifies whether the network will allow self-loops (edges that connect a node to itself).
        Attempting to add a self-loop to a network that does not allow them will throw an UnsupportedOperationException.
        
        The default value is `False`.
        """
        ...


    def expectedNodeCount(self, expectedNodeCount: int) -> "NetworkBuilder"["N", "E"]:
        """
        Specifies the expected number of nodes in the network.

        Raises
        - IllegalArgumentException: if `expectedNodeCount` is negative
        """
        ...


    def expectedEdgeCount(self, expectedEdgeCount: int) -> "NetworkBuilder"["N", "E"]:
        """
        Specifies the expected number of edges in the network.

        Raises
        - IllegalArgumentException: if `expectedEdgeCount` is negative
        """
        ...


    def nodeOrder(self, nodeOrder: "ElementOrder"["N1"]) -> "NetworkBuilder"["N1", "E"]:
        """
        Specifies the order of iteration for the elements of Network.nodes().
        
        The default value is ElementOrder.insertion() insertion order.
        """
        ...


    def edgeOrder(self, edgeOrder: "ElementOrder"["E1"]) -> "NetworkBuilder"["N", "E1"]:
        """
        Specifies the order of iteration for the elements of Network.edges().
        
        The default value is ElementOrder.insertion() insertion order.
        """
        ...


    def build(self) -> "MutableNetwork"["N1", "E1"]:
        """
        Returns an empty MutableNetwork with the properties of this NetworkBuilder.
        """
        ...



"""
Python module generated from Java source file org.joml.SimplexNoise

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class SimplexNoise:
    """
    A simplex noise algorithm for 2D, 3D and 4D input.
    
    It was originally authored by Stefan Gustavson.
    
    The original implementation can be found here: <a
    href="http://staffwww.itn.liu.se/~stegu/simplexnoise/SimplexNoise.java">http://http://staffwww.itn.liu.se/</a>.
    """

    @staticmethod
    def noise(x: float, y: float) -> float:
        """
        Compute 2D simplex noise for the given input vector `(x, y)`.
        
        The result is in the range `[-1..+1]`.

        Arguments
        - x: the x coordinate
        - y: the y coordinate

        Returns
        - the noise value (within `[-1..+1]`)
        """
        ...


    @staticmethod
    def noise(x: float, y: float, z: float) -> float:
        """
        Compute 3D simplex noise for the given input vector `(x, y, z)`.
        
        The result is in the range `[-1..+1]`.

        Arguments
        - x: the x coordinate
        - y: the y coordinate
        - z: the z coordinate

        Returns
        - the noise value (within `[-1..+1]`)
        """
        ...


    @staticmethod
    def noise(x: float, y: float, z: float, w: float) -> float:
        """
        Compute 4D simplex noise for the given input vector `(x, y, z, w)`.
        
        The result is in the range `[-1..+1]`.

        Arguments
        - x: the x coordinate
        - y: the y coordinate
        - z: the z coordinate
        - w: the w coordinate

        Returns
        - the noise value (within `[-1..+1]`)
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerLoginEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.net import InetAddress
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerLoginEvent(PlayerEvent):
    """
    Stores details for players attempting to log in.
    
    Note that this event is called *early* in the player initialization
    process. It is recommended that most options involving the Player
    *entity* be postponed to the PlayerJoinEvent instead.
    """

    def __init__(self, player: "Player", hostname: str, address: "InetAddress", realAddress: "InetAddress"):
        """
        This constructor defaults message to an empty string, and result to
        ALLOWED

        Arguments
        - player: The Player for this event
        - hostname: The hostname that was used to connect to the server
        - address: The address the player used to connect, provided for
            timing issues
        - realAddress: the actual, unspoofed connecting address
        """
        ...


    def __init__(self, player: "Player", hostname: str, address: "InetAddress"):
        """
        This constructor defaults message to an empty string, and result to
        ALLOWED

        Arguments
        - player: The Player for this event
        - hostname: The hostname that was used to connect to the server
        - address: The address the player used to connect, provided for
            timing issues
        """
        ...


    def __init__(self, player: "Player", hostname: str, address: "InetAddress", result: "Result", message: str, realAddress: "InetAddress"):
        """
        This constructor pre-configures the event with a result and message

        Arguments
        - player: The Player for this event
        - hostname: The hostname that was used to connect to the server
        - address: The address the player used to connect, provided for
            timing issues
        - result: The result status for this event
        - message: The message to be displayed if result denies login
        - realAddress: the actual, unspoofed connecting address
        """
        ...


    def getResult(self) -> "Result":
        """
        Gets the current result of the login, as an enum

        Returns
        - Current Result of the login
        """
        ...


    def setResult(self, result: "Result") -> None:
        """
        Sets the new result of the login, as an enum

        Arguments
        - result: New result to set
        """
        ...


    def getKickMessage(self) -> str:
        """
        Gets the current kick message that will be used if getResult() !=
        Result.ALLOWED

        Returns
        - Current kick message
        """
        ...


    def setKickMessage(self, message: str) -> None:
        """
        Sets the kick message to display if getResult() != Result.ALLOWED

        Arguments
        - message: New kick message
        """
        ...


    def getHostname(self) -> str:
        """
        Gets the hostname that the player used to connect to the server, or
        blank if unknown

        Returns
        - The hostname
        """
        ...


    def allow(self) -> None:
        """
        Allows the player to log in
        """
        ...


    def disallow(self, result: "Result", message: str) -> None:
        """
        Disallows the player from logging in, with the given reason

        Arguments
        - result: New result for disallowing the player
        - message: Kick message to display to the user
        """
        ...


    def getAddress(self) -> "InetAddress":
        """
        Gets the InetAddress for the Player associated with this event.
        This method is provided as a workaround for player.getAddress()
        returning null during PlayerLoginEvent.

        Returns
        - The address for this player. For legacy compatibility, this may
            be null.
        """
        ...


    def getRealAddress(self) -> "InetAddress":
        """
        Gets the connection address of this player, regardless of whether it has
        been spoofed or not.

        Returns
        - the player's connection address

        See
        - .getAddress()
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class Result(Enum):
        """
        Basic kick reasons for communicating to plugins
        """

        ALLOWED = 0
        """
        The player is allowed to log in
        """
        KICK_FULL = 1
        """
        The player is not allowed to log in, due to the server being full
        """
        KICK_BANNED = 2
        """
        The player is not allowed to log in, due to them being banned
        """
        KICK_WHITELIST = 3
        """
        The player is not allowed to log in, due to them not being on the
        white list
        """
        KICK_OTHER = 4
        """
        The player is not allowed to log in, for reasons undefined
        """



"""
Python module generated from Java source file org.bukkit.inventory.MenuType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.entity import HumanEntity
from org.bukkit.inventory import *
from org.bukkit.inventory.view import AnvilView
from org.bukkit.inventory.view import BeaconView
from org.bukkit.inventory.view import BrewingStandView
from org.bukkit.inventory.view import CrafterView
from org.bukkit.inventory.view import EnchantmentView
from org.bukkit.inventory.view import FurnaceView
from org.bukkit.inventory.view import LecternView
from org.bukkit.inventory.view import LoomView
from org.bukkit.inventory.view import MerchantView
from org.bukkit.inventory.view import StonecutterView
from org.bukkit.inventory.view.builder import InventoryViewBuilder
from org.bukkit.inventory.view.builder import LocationInventoryViewBuilder
from org.bukkit.inventory.view.builder import MerchantInventoryViewBuilder
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class MenuType(Keyed, RegistryAware):
    """
    Represents different kinds of views, also known as menus, which can be
    created and viewed by the player.
    """

    GENERIC_9X1 = get("generic_9x1")
    """
    A MenuType which represents a chest with 1 row.
    """
    GENERIC_9X2 = get("generic_9x2")
    """
    A MenuType which represents a chest with 2 rows.
    """
    GENERIC_9X3 = get("generic_9x3")
    """
    A MenuType which represents a chest with 3 rows.
    """
    GENERIC_9X4 = get("generic_9x4")
    """
    A MenuType which represents a chest with 4 rows.
    """
    GENERIC_9X5 = get("generic_9x5")
    """
    A MenuType which represents a chest with 5 rows.
    """
    GENERIC_9X6 = get("generic_9x6")
    """
    A MenuType which represents a chest with 6 rows.
    """
    GENERIC_3X3 = get("generic_3x3")
    """
    A MenuType which represents a dispenser/dropper like menu with 3 columns
    and 3 rows.
    """
    CRAFTER_3X3 = get("crafter_3x3")
    """
    A MenuType which represents a crafter
    """
    ANVIL = get("anvil")
    """
    A MenuType which represents an anvil.
    """
    BEACON = get("beacon")
    """
    A MenuType which represents a beacon.
    """
    BLAST_FURNACE = get("blast_furnace")
    """
    A MenuType which represents a blast furnace.
    """
    BREWING_STAND = get("brewing_stand")
    """
    A MenuType which represents a brewing stand.
    """
    CRAFTING = get("crafting")
    """
    A MenuType which represents a crafting table.
    """
    ENCHANTMENT = get("enchantment")
    """
    A MenuType which represents an enchantment table.
    """
    FURNACE = get("furnace")
    """
    A MenuType which represents a furnace.
    """
    GRINDSTONE = get("grindstone")
    """
    A MenuType which represents a grindstone.
    """
    HOPPER = get("hopper")
    """
    A MenuType which represents a hopper.
    """
    LECTERN = get("lectern")
    """
    A MenuType which represents a lectern, a book like view.
    """
    LOOM = get("loom")
    """
    A MenuType which represents a loom.
    """
    MERCHANT = get("merchant")
    """
    A MenuType which represents a merchant.
    """
    SHULKER_BOX = get("shulker_box")
    """
    A MenuType which represents a shulker box.
    """
    SMITHING = get("smithing")
    """
    A MenuType which represents a stonecutter.
    """
    SMOKER = get("smoker")
    """
    A MenuType which represents a smoker.
    """
    CARTOGRAPHY_TABLE = get("cartography_table")
    """
    A MenuType which represents a cartography table.
    """
    STONECUTTER = get("stonecutter")
    """
    A MenuType which represents a stonecutter.
    """


    def typed(self) -> "MenuType.Typed"["InventoryView", "InventoryViewBuilder"["InventoryView"]]:
        """
        Yields this MenuType as a typed version of itself with a plain
        InventoryView representing it.

        Returns
        - the typed MenuType.
        """
        ...


    def typed(self, viewClass: type["V"]) -> "MenuType.Typed"["V", "B"]:
        """
        Yields this MenuType as a typed version of itself with a specific
        InventoryView representing it.
        
        Type `<V>`: the generic type of the InventoryView to get this MenuType
        with
        
        Type `<B>`: the generic type of the InventoryViewBuilder to get this
        MenuType with

        Arguments
        - viewClass: the class type of the InventoryView to type this
        InventoryView with.

        Returns
        - the typed MenuType

        Raises
        - IllegalArgumentException: if the provided viewClass cannot be
        typed to this MenuType
        """
        ...


    def getInventoryViewClass(self) -> type["InventoryView"]:
        """
        Gets the InventoryView class of this MenuType.

        Returns
        - the InventoryView class of this MenuType
        """
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    @staticmethod
    def get(key: str) -> "T":
        ...



from .UsingToStringOrdering import UsingToStringOrdering
from .UnmodifiableSortedMultiset import UnmodifiableSortedMultiset
from .UnmodifiableListIterator import UnmodifiableListIterator
from .UnmodifiableIterator import UnmodifiableIterator
from .TreeTraverser import TreeTraverser
from .TreeRangeSet import TreeRangeSet
from .TreeRangeMap import TreeRangeMap
from .TreeMultiset import TreeMultiset
from .TreeMultimap import TreeMultimap
from .TreeBasedTable import TreeBasedTable
from .TransformedListIterator import TransformedListIterator
from .TransformedIterator import TransformedIterator
from .TopKSelector import TopKSelector
from .Tables import Tables
from .TableCollectors import TableCollectors
from .Table import Table
from .Synchronized import Synchronized
from .Streams import Streams
from .StandardTable import StandardTable
from .StandardRowSortedTable import StandardRowSortedTable
from .SparseImmutableTable import SparseImmutableTable
from .SortedSetMultimap import SortedSetMultimap
from .SortedMultisets import SortedMultisets
from .SortedMultisetBridge import SortedMultisetBridge
from .SortedMultiset import SortedMultiset
from .SortedMapDifference import SortedMapDifference
from .SortedLists import SortedLists
from .SortedIterables import SortedIterables
from .SortedIterable import SortedIterable
from .SingletonImmutableTable import SingletonImmutableTable
from .SingletonImmutableSet import SingletonImmutableSet
from .SingletonImmutableList import SingletonImmutableList
from .SingletonImmutableBiMap import SingletonImmutableBiMap
from .Sets import Sets
from .SetMultimap import SetMultimap
from .Serialization import Serialization
from .RowSortedTable import RowSortedTable
from .ReverseOrdering import ReverseOrdering
from .ReverseNaturalOrdering import ReverseNaturalOrdering
from .RegularImmutableTable import RegularImmutableTable
from .RegularImmutableSortedSet import RegularImmutableSortedSet
from .RegularImmutableSortedMultiset import RegularImmutableSortedMultiset
from .RegularImmutableSet import RegularImmutableSet
from .RegularImmutableMultiset import RegularImmutableMultiset
from .RegularImmutableMap import RegularImmutableMap
from .RegularImmutableList import RegularImmutableList
from .RegularImmutableBiMap import RegularImmutableBiMap
from .RegularImmutableAsList import RegularImmutableAsList
from .RegularContiguousSet import RegularContiguousSet
from .RangeSet import RangeSet
from .RangeMap import RangeMap
from .RangeGwtSerializationDependencies import RangeGwtSerializationDependencies
from .Range import Range
from .Queues import Queues
from .Platform import Platform
from .PeekingIterator import PeekingIterator
from .ParametricNullness import ParametricNullness
from .Ordering import Ordering
from .ObjectArrays import ObjectArrays
from .NullsLastOrdering import NullsLastOrdering
from .NullsFirstOrdering import NullsFirstOrdering
from .NullnessCasts import NullnessCasts
from .NaturalOrdering import NaturalOrdering
from .MutableClassToInstanceMap import MutableClassToInstanceMap
from .Multisets import Multisets
from .Multiset import Multiset
from .Multimaps import Multimaps
from .MultimapBuilder import MultimapBuilder
from .Multimap import Multimap
from .MoreCollectors import MoreCollectors
from .MinMaxPriorityQueue import MinMaxPriorityQueue
from .Maps import Maps
from .MapMakerInternalMap import MapMakerInternalMap
from .MapMaker import MapMaker
from .MapDifference import MapDifference
from .Lists import Lists
from .ListMultimap import ListMultimap
from .LinkedListMultimap import LinkedListMultimap
from .LinkedHashMultiset import LinkedHashMultiset
from .LinkedHashMultimapGwtSerializationDependencies import LinkedHashMultimapGwtSerializationDependencies
from .LinkedHashMultimap import LinkedHashMultimap
from .LexicographicalOrdering import LexicographicalOrdering
from .JdkBackedImmutableSet import JdkBackedImmutableSet
from .JdkBackedImmutableMultiset import JdkBackedImmutableMultiset
from .JdkBackedImmutableMap import JdkBackedImmutableMap
from .JdkBackedImmutableBiMap import JdkBackedImmutableBiMap
from .Iterators import Iterators
from .Iterables import Iterables
from .Interners import Interners
from .Interner import Interner
from .IndexedImmutableSet import IndexedImmutableSet
from .ImmutableTable import ImmutableTable
from .ImmutableSortedSet import ImmutableSortedSet
from .ImmutableSortedMultiset import ImmutableSortedMultiset
from .ImmutableSortedMap import ImmutableSortedMap
from .ImmutableSortedAsList import ImmutableSortedAsList
from .ImmutableSetMultimap import ImmutableSetMultimap
from .ImmutableSet import ImmutableSet
from .ImmutableRangeSet import ImmutableRangeSet
from .ImmutableRangeMap import ImmutableRangeMap
from .ImmutableMultisetGwtSerializationDependencies import ImmutableMultisetGwtSerializationDependencies
from .ImmutableMultiset import ImmutableMultiset
from .ImmutableMultimap import ImmutableMultimap
from .ImmutableMapValues import ImmutableMapValues
from .ImmutableMapKeySet import ImmutableMapKeySet
from .ImmutableMapEntrySet import ImmutableMapEntrySet
from .ImmutableMapEntry import ImmutableMapEntry
from .ImmutableMap import ImmutableMap
from .ImmutableListMultimap import ImmutableListMultimap
from .ImmutableList import ImmutableList
from .ImmutableEnumSet import ImmutableEnumSet
from .ImmutableEnumMap import ImmutableEnumMap
from .ImmutableEntry import ImmutableEntry
from .ImmutableCollection import ImmutableCollection
from .ImmutableClassToInstanceMap import ImmutableClassToInstanceMap
from .ImmutableBiMap import ImmutableBiMap
from .ImmutableAsList import ImmutableAsList
from .IgnoreJRERequirement import IgnoreJRERequirement
from .Hashing import Hashing
from .HashMultiset import HashMultiset
from .HashMultimapGwtSerializationDependencies import HashMultimapGwtSerializationDependencies
from .HashMultimap import HashMultimap
from .HashBiMap import HashBiMap
from .HashBasedTable import HashBasedTable
from .GwtTransient import GwtTransient
from .GeneralRange import GeneralRange
from .ForwardingTable import ForwardingTable
from .ForwardingSortedSetMultimap import ForwardingSortedSetMultimap
from .ForwardingSortedSet import ForwardingSortedSet
from .ForwardingSortedMultiset import ForwardingSortedMultiset
from .ForwardingSortedMap import ForwardingSortedMap
from .ForwardingSetMultimap import ForwardingSetMultimap
from .ForwardingSet import ForwardingSet
from .ForwardingQueue import ForwardingQueue
from .ForwardingObject import ForwardingObject
from .ForwardingNavigableSet import ForwardingNavigableSet
from .ForwardingNavigableMap import ForwardingNavigableMap
from .ForwardingMultiset import ForwardingMultiset
from .ForwardingMultimap import ForwardingMultimap
from .ForwardingMapEntry import ForwardingMapEntry
from .ForwardingMap import ForwardingMap
from .ForwardingListMultimap import ForwardingListMultimap
from .ForwardingListIterator import ForwardingListIterator
from .ForwardingList import ForwardingList
from .ForwardingIterator import ForwardingIterator
from .ForwardingImmutableSet import ForwardingImmutableSet
from .ForwardingImmutableMap import ForwardingImmutableMap
from .ForwardingImmutableList import ForwardingImmutableList
from .ForwardingImmutableCollection import ForwardingImmutableCollection
from .ForwardingDeque import ForwardingDeque
from .ForwardingConcurrentMap import ForwardingConcurrentMap
from .ForwardingCollection import ForwardingCollection
from .ForwardingBlockingDeque import ForwardingBlockingDeque
from .FluentIterable import FluentIterable
from .FilteredSetMultimap import FilteredSetMultimap
from .FilteredMultimapValues import FilteredMultimapValues
from .FilteredMultimap import FilteredMultimap
from .FilteredKeySetMultimap import FilteredKeySetMultimap
from .FilteredKeyMultimap import FilteredKeyMultimap
from .FilteredKeyListMultimap import FilteredKeyListMultimap
from .FilteredEntrySetMultimap import FilteredEntrySetMultimap
from .FilteredEntryMultimap import FilteredEntryMultimap
from .ExplicitOrdering import ExplicitOrdering
from .EvictingQueue import EvictingQueue
from .EnumMultiset import EnumMultiset
from .EnumHashBiMap import EnumHashBiMap
from .EnumBiMap import EnumBiMap
from .EmptyImmutableSetMultimap import EmptyImmutableSetMultimap
from .EmptyImmutableListMultimap import EmptyImmutableListMultimap
from .EmptyContiguousSet import EmptyContiguousSet
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .DiscreteDomain import DiscreteDomain
from .DescendingMultiset import DescendingMultiset
from .DescendingImmutableSortedSet import DescendingImmutableSortedSet
from .DescendingImmutableSortedMultiset import DescendingImmutableSortedMultiset
from .DenseImmutableTable import DenseImmutableTable
from .Cut import Cut
from .Count import Count
from .ContiguousSet import ContiguousSet
from .ConsumingQueueIterator import ConsumingQueueIterator
from .ConcurrentHashMultiset import ConcurrentHashMultiset
from .ComputationException import ComputationException
from .CompoundOrdering import CompoundOrdering
from .ComparisonChain import ComparisonChain
from .Comparators import Comparators
from .ComparatorOrdering import ComparatorOrdering
from .CompactLinkedHashSet import CompactLinkedHashSet
from .CompactLinkedHashMap import CompactLinkedHashMap
from .CompactHashing import CompactHashing
from .CompactHashSet import CompactHashSet
from .CompactHashMap import CompactHashMap
from .Collections2 import Collections2
from .CollectSpliterators import CollectSpliterators
from .CollectPreconditions import CollectPreconditions
from .CollectCollectors import CollectCollectors
from .ClassToInstanceMap import ClassToInstanceMap
from .CartesianList import CartesianList
from .ByFunctionOrdering import ByFunctionOrdering
from .BoundType import BoundType
from .BiMap import BiMap
from .BaseImmutableMultimap import BaseImmutableMultimap
from .ArrayTable import ArrayTable
from .ArrayListMultimapGwtSerializationDependencies import ArrayListMultimapGwtSerializationDependencies
from .ArrayListMultimap import ArrayListMultimap
from .AllEqualOrdering import AllEqualOrdering
from .AbstractTable import AbstractTable
from .AbstractSortedSetMultimap import AbstractSortedSetMultimap
from .AbstractSortedMultiset import AbstractSortedMultiset
from .AbstractSortedKeySortedSetMultimap import AbstractSortedKeySortedSetMultimap
from .AbstractSetMultimap import AbstractSetMultimap
from .AbstractSequentialIterator import AbstractSequentialIterator
from .AbstractRangeSet import AbstractRangeSet
from .AbstractNavigableMap import AbstractNavigableMap
from .AbstractMultiset import AbstractMultiset
from .AbstractMultimap import AbstractMultimap
from .AbstractMapEntry import AbstractMapEntry
from .AbstractMapBasedMultiset import AbstractMapBasedMultiset
from .AbstractMapBasedMultimap import AbstractMapBasedMultimap
from .AbstractListMultimap import AbstractListMultimap
from .AbstractIterator import AbstractIterator
from .AbstractIndexedListIterator import AbstractIndexedListIterator
from .AbstractBiMap import AbstractBiMap


"""
Python module generated from Java source file com.google.common.base.MoreObjects

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.lang.reflect import Array
from java.util import Arrays
from java.util import OptionalDouble
from java.util import OptionalInt
from java.util import OptionalLong
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class MoreObjects:
    """
    Helper functions that operate on any `Object`, and are not already provided in java.util.Objects.
    
    See the Guava User Guide on <a
    href="https://github.com/google/guava/wiki/CommonObjectUtilitiesExplained">writing `Object`
    methods with `MoreObjects`</a>.

    Author(s)
    - Laurence Gonsalves

    Since
    - 18.0 (since 2.0 as `Objects`)
    """

    @staticmethod
    def firstNonNull(first: "T", second: "T") -> "T":
        """
        Returns the first of two given parameters that is not `null`, if either is, or otherwise
        throws a NullPointerException.
        
        To find the first non-null element in an iterable, use `Iterables.find(iterable,
        Predicates.notNull())`. For varargs, use `Iterables.find(Arrays.asList(a, b, c, ...),
        Predicates.notNull())`, static importing as necessary.
        
        **Note:** if `first` is represented as an Optional, this can be
        accomplished with Optional.or(Object) first.or(second). That approach also allows for
        lazy evaluation of the fallback instance, using Optional.or(Supplier)
        first.or(supplier).
        
        **Java 9 users:** use `java.util.Objects.requireNonNullElse(first, second)`
        instead.

        Returns
        - `first` if it is non-null; otherwise `second` if it is non-null

        Raises
        - NullPointerException: if both `first` and `second` are null

        Since
        - 18.0 (since 3.0 as `Objects.firstNonNull()`).
        """
        ...


    @staticmethod
    def toStringHelper(self: "Object") -> "ToStringHelper":
        """
        Creates an instance of ToStringHelper.
        
        This is helpful for implementing Object.toString(). Specification by example:
        
        ````// Returns "ClassName{`"
        MoreObjects.toStringHelper(this)
            .toString();
        
        // Returns "ClassName{x=1}"
        MoreObjects.toStringHelper(this)
            .add("x", 1)
            .toString();
        
        // Returns "MyObject{x=1}"
        MoreObjects.toStringHelper("MyObject")
            .add("x", 1)
            .toString();
        
        // Returns "ClassName{x=1, y=foo}"
        MoreObjects.toStringHelper(this)
            .add("x", 1)
            .add("y", "foo")
            .toString();
        
        // Returns "ClassName{x=1}"
        MoreObjects.toStringHelper(this)
            .omitNullValues()
            .add("x", 1)
            .add("y", null)
            .toString();
        }```
        
        Note that in GWT, class names are often obfuscated.

        Arguments
        - self: the object to generate the string for (typically `this`), used only for its
            class name

        Since
        - 18.0 (since 2.0 as `Objects.toStringHelper()`).
        """
        ...


    @staticmethod
    def toStringHelper(clazz: type[Any]) -> "ToStringHelper":
        """
        Creates an instance of ToStringHelper in the same manner as .toStringHelper(Object), but using the simple name of `clazz` instead of using an
        instance's Object.getClass().
        
        Note that in GWT, class names are often obfuscated.

        Arguments
        - clazz: the Class of the instance

        Since
        - 18.0 (since 7.0 as `Objects.toStringHelper()`).
        """
        ...


    @staticmethod
    def toStringHelper(className: str) -> "ToStringHelper":
        """
        Creates an instance of ToStringHelper in the same manner as .toStringHelper(Object), but using `className` instead of using an instance's Object.getClass().

        Arguments
        - className: the name of the instance type

        Since
        - 18.0 (since 7.0 as `Objects.toStringHelper()`).
        """
        ...


    class ToStringHelper:
        """
        Support class for MoreObjects.toStringHelper.

    Author(s)
        - Jason Lee

        Since
        - 18.0 (since 2.0 as `Objects.ToStringHelper`).
        """

        def omitNullValues(self) -> "ToStringHelper":
            """
            Configures the ToStringHelper so .toString() will ignore properties with null
            value. The order of calling this method, relative to the `add()`/`addValue()`
            methods, is not significant.

            Since
            - 18.0 (since 12.0 as `Objects.ToStringHelper.omitNullValues()`).
            """
            ...


        def add(self, name: str, value: "Object") -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format. If `value`
            is `null`, the string `"null"` is used, unless .omitNullValues() is
            called, in which case this name/value pair will not be added.
            """
            ...


        def add(self, name: str, value: bool) -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.add()`).
            """
            ...


        def add(self, name: str, value: str) -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.add()`).
            """
            ...


        def add(self, name: str, value: float) -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.add()`).
            """
            ...


        def add(self, name: str, value: float) -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.add()`).
            """
            ...


        def add(self, name: str, value: int) -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.add()`).
            """
            ...


        def add(self, name: str, value: int) -> "ToStringHelper":
            """
            Adds a name/value pair to the formatted output in `name=value` format.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.add()`).
            """
            ...


        def addValue(self, value: "Object") -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, Object) instead and give value a
            readable name.
            """
            ...


        def addValue(self, value: bool) -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, boolean) instead and give value a
            readable name.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.addValue()`).
            """
            ...


        def addValue(self, value: str) -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, char) instead and give value a
            readable name.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.addValue()`).
            """
            ...


        def addValue(self, value: float) -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, double) instead and give value a
            readable name.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.addValue()`).
            """
            ...


        def addValue(self, value: float) -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, float) instead and give value a
            readable name.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.addValue()`).
            """
            ...


        def addValue(self, value: int) -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, int) instead and give value a
            readable name.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.addValue()`).
            """
            ...


        def addValue(self, value: int) -> "ToStringHelper":
            """
            Adds an unnamed value to the formatted output.
            
            It is strongly encouraged to use .add(String, long) instead and give value a
            readable name.

            Since
            - 18.0 (since 11.0 as `Objects.ToStringHelper.addValue()`).
            """
            ...


        def toString(self) -> str:
            """
            Returns a string in the format specified by MoreObjects.toStringHelper(Object).
            
            After calling this method, you can keep adding more properties to later call toString()
            again and get a more complete representation of the same object; but properties cannot be
            removed, so this only allows limited reuse of the helper instance. The helper allows
            duplication of properties (multiple name/value pairs with the same name can be added).
            """
            ...



"""
Python module generated from Java source file org.joml.RoundingMode

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class RoundingMode:
    """
    Rounding modes.

    Author(s)
    - Kai Burjack
    """

    TRUNCATE = 0
    """
    Discards the fractional part.
    """
    CEILING = 1
    """
    Round towards positive infinity.
    """
    FLOOR = 2
    """
    Round towards negative infinity.
    """
    HALF_EVEN = 3
    """
    Round towards the nearest neighbor. If both neighbors are equidistant, round
    towards the even neighbor.
    """
    HALF_DOWN = 4
    """
    Round towards the nearest neighbor. If both neighbors are equidistant, round
    down.
    """
    HALF_UP = 5
    """
    Round towards the nearest neighbor. If both neighbors are equidistant, round
    up.
    """
